@@ -28,7 +28,7 @@
 #
 # Run as:
 #
-#    sudo docker run -it --rm \
+#    sudo docker run -it --rm --pull=always \
 #         -v /tmp/artifact:/artifact \
 #         -v /home/jfreegman/git/toxic:/toxic \
 #         amd64/alpine:latest \
@@ -87,6 +87,13 @@ set -x
 # Use all cores for building
 MAKEFLAGS=j$(nproc)
 export MAKEFLAGS
+# TODO(nurupo): Once GCC 14 comes out, switch to using the new -fhardened
+CFLAGS="-ftrivial-auto-var-init=zero -fPIE -pie -Wl,-z,relro,-z,now -fstack-protector-strong -fstack-clash-protection"
+if [ "$(uname -m)" == "x86_64" ]
+then
+  CFLAGS="$CFLAGS -fcf-protection=full"
+fi
+export CFLAGS
 
 check_sha256()
 {
@@ -136,10 +143,10 @@ mkdir -p "$BUILD_DIR"
 cd "$BUILD_DIR"
 
 # The git hash of the c-toxcore version we're using
-TOXCORE_VERSION="0ec4978de51a113223c56d44dfe0a23c184e4c6d"
+TOXCORE_VERSION="v0.2.19"
 
 # The sha256sum of the c-toxcore tarball for TOXCORE_VERSION
-TOXCORE_HASH="b1a3fef4493d709508e9e02876a3239abd5c395c618016d67e1cba817d96d19c"
+TOXCORE_HASH="b770206a5ce59734279ac79f533bebcc8c8ff1d452cdd6628289a49b9200f428"
 
 TOXCORE_FILENAME="c-toxcore-$TOXCORE_VERSION.tar.gz"
 
@@ -180,20 +187,57 @@ cmake --build _build --target install
 # location with SSL_CERT_FILE env variable.
 cd "$BUILD_DIR"
 
-CURL_VERSION="8.5.0"
-CURL_HASH="05fc17ff25b793a437a0906e0484b82172a9f4de02be5ed447e0cab8c3475add"
-CURL_FILENAME="curl-$CURL_VERSION.tar.gz"
+CURL_VERSION="8.7.1"
+CURL_HASH="6fea2aac6a4610fbd0400afb0bcddbe7258a64c63f1f68e5855ebc0c659710cd"
+CURL_FILENAME="curl-$CURL_VERSION.tar.xz"
 
 wget --timeout=10 -O "$CURL_FILENAME" "https://curl.haxx.se/download/$CURL_FILENAME"
 check_sha256 "$CURL_HASH" "$CURL_FILENAME"
-tar -xf curl*.tar.gz
-rm curl*.tar.gz
+tar -xf "$CURL_FILENAME"
+rm "$CURL_FILENAME"
 cd curl*
 
+# TODO(nurupo): backport of https://github.com/curl/curl/commit/4b42cda3df85419328ba8c9160a3e8306605d094
+#               remove once that commit makes it into a curl release
+sed -i 's/man_MANS = $(MANPAGE)/@BUILD_DOCS_TRUE@man_MANS = $(MANPAGE)/g' docs/cmdline-opts/Makefile.in
 ./configure \
   --prefix="$BUILD_DIR/prefix-curl" \
   --disable-shared \
   --enable-static \
+  --disable-ftp \
+  --disable-file \
+  --disable-ldap \
+  --disable-ldaps \
+  --disable-rtsp \
+  --disable-proxy \
+  --disable-dict \
+  --disable-telnet \
+  --disable-tftp \
+  --disable-pop3 \
+  --disable-imap \
+  --disable-smb \
+  --disable-smtp \
+  --disable-gopher \
+  --disable-mqtt \
+  --disable-docs \
+  --disable-manual \
+  --disable-libcurl-option \
+  --disable-sspi \
+  --disable-basic-auth \
+  --disable-bearer-auth \
+  --disable-digest-auth \
+  --disable-kerberos-auth \
+  --disable-negotiate-auth \
+  --disable-aws \
+  --disable-ntlm \
+  --disable-ntlm-wb \
+  --disable-tls-srp \
+  --disable-unix-sockets \
+  --disable-cookies \
+  --disable-http-auth \
+  --disable-bindlocal \
+  --disable-netrc \
+  --disable-websockets \
   --without-ca-bundle \
   --without-ca-path \
   --with-ca-fallback \
@@ -210,6 +254,8 @@ cd "$BUILD_DIR"
 cp -a "$TOXIC_SRC_DIR" toxic
 cd toxic
 
+git config --global --add safe.directory "$PWD"
+
 if [ -z "$(git describe --tags --exact-match HEAD)" ]
 then
   set +x
@@ -226,7 +272,7 @@ fi
 sed -i 's|pkg-config|pkg-config --static|' cfg/global_vars.mk
 sed -i 's|<limits.h|<linux/limits.h|' src/*
 
-CFLAGS="-static" PKG_CONFIG_PATH="$BUILD_DIR/prefix-toxcore/lib64/pkgconfig:$BUILD_DIR/prefix-toxcore/lib/pkgconfig:$BUILD_DIR/prefix-curl/lib/pkgconfig" PREFIX="$BUILD_DIR/prefix-toxic" make \
+CFLAGS="$CFLAGS -static" PKG_CONFIG_PATH="$BUILD_DIR/prefix-toxcore/lib64/pkgconfig:$BUILD_DIR/prefix-toxcore/lib/pkgconfig:$BUILD_DIR/prefix-curl/lib/pkgconfig" PREFIX="$BUILD_DIR/prefix-toxic" make \
   DISABLE_X11=1 \
   DISABLE_AV=1 \
   DISABLE_SOUND_NOTIFY=1 \
@@ -251,16 +297,27 @@ mv "$PREPARE_ARTIFACT_DIR/toxic.conf.example" "$PREPARE_ARTIFACT_DIR/toxic.conf"
 
 cp -aL /usr/share/terminfo "$PREPARE_ARTIFACT_DIR"
 
-echo "A minimal statically compiled Toxic.
-Doesn't support X11 integration, video/audio calls, desktop & sound
-notifications, QR codes and Python scripting.
-However, it is rather portable.
+TOXIC_GIT_TAG="$(git -C "$BUILD_DIR/toxic" describe --tags --abbrev=0 || true)"
+# GitHub PRs do shallow clones without tags
+if [ -z "$TOXIC_GIT_TAG" ]
+then
+  git -C "$BUILD_DIR/toxic" fetch origin --unshallow
+  TOXIC_GIT_TAG="$(git -C "$BUILD_DIR/toxic" describe --tags --abbrev=0)"
+fi
+TOXIC_GIT_COMMIT="$(git -C "$BUILD_DIR/toxic" rev-parse HEAD)"
+TOXIC_GIT_REV="$(git -C "$BUILD_DIR/toxic" rev-list HEAD --count)"
+TOXIC_VERSION="${TOXIC_GIT_TAG}_r${TOXIC_GIT_REV} ($TOXIC_GIT_COMMIT)"
 
-Toxic $(git -C "$BUILD_DIR/toxic" describe --tags --exact-match HEAD) ($(git -C "$BUILD_DIR/toxic" rev-parse HEAD))
+echo "A minimal statically compiled Toxic. Doesn't support X11 integration,
+video/audio calls, desktop & sound notifications, QR codes and Python
+scripting. However, it is rather portable.
 
-Build date time: $(TZ=UTC date +"%Y-%m-%dT%H:%M:%S%z")
+Toxic $TOXIC_VERSION
+
+Build date-time: $(TZ=UTC date +"%Y-%m-%dT%H:%M:%S%z")
 
 OS:
+$ cat /etc/os-release
 $(cat /etc/os-release)
 
 List of self-built software statically compiled into Toxic:
@@ -268,9 +325,11 @@ libcurl $CURL_VERSION
 libtoxcore $TOXCORE_VERSION
 
 List of OS-packaged software statically compiled into Toxic:
+$ apk list -I | grep 'static' | sort -i
 $(apk list -I | grep 'static' | sort -i)
 
 List of all packages installed during the build:
+$ apk list -I | sort -i
 $(apk list -I | sort -i)" > "$PREPARE_ARTIFACT_DIR/build_info"
 
 echo '#!/usr/bin/env sh
