@@ -1,23 +1,9 @@
 /*  audio_call.c
  *
+ *  Copyright (C) 2014-2024 Toxic All Rights Reserved.
  *
- *  Copyright (C) 2024 Toxic All Rights Reserved.
- *
- *  This file is part of Toxic.
- *
- *  Toxic is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
- *
- *  Toxic is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with Toxic.  If not, see <http://www.gnu.org/licenses/>.
- *
+ *  This file is part of Toxic. Toxic is free software licensed
+ *  under the GNU General Public License 3.0.
  */
 
 #include "audio_call.h"
@@ -319,6 +305,11 @@ void on_call(ToxAV *av, uint32_t friend_number, bool audio_enabled, bool video_e
 
     Toxic *toxic = (Toxic *) user_data;
 
+    if (friend_number >= CallControl.max_calls) {
+        fprintf(stderr, "Failed to receive call: Insufficient memory\n");
+        return;
+    }
+
     Call *call = &CallControl.calls[friend_number];
     init_call(call);
 
@@ -345,6 +336,11 @@ void on_call_state(ToxAV *av, uint32_t friend_number, uint32_t state, void *user
 
     UNUSED_VAR(av);
 
+    if (friend_number >= CallControl.max_calls) {
+        fprintf(stderr, "Failed to handle call state: Insufficient memory\n");
+        return;
+    }
+
     Call *call = &CallControl.calls[friend_number];
 
     if (call->status == cs_None) {
@@ -412,6 +408,10 @@ void audio_bit_rate_callback(ToxAV *av, uint32_t friend_number, uint32_t audio_b
 {
     UNUSED_VAR(user_data);
 
+    if (friend_number >= CallControl.max_calls) {
+        return;
+    }
+
     Call *call = &CallControl.calls[friend_number];
     call->audio_bit_rate = audio_bit_rate;
     toxav_audio_set_bit_rate(av, friend_number, audio_bit_rate, NULL);
@@ -419,90 +419,137 @@ void audio_bit_rate_callback(ToxAV *av, uint32_t friend_number, uint32_t audio_b
 
 void callback_recv_invite(Toxic *toxic, uint32_t friend_number)
 {
-    if (toxic == NULL) {
-        return;
-    }
-
     if (friend_number >= Friends.max_idx) {
         return;
     }
 
-    if (Friends.list[friend_number].chatwin == -1) {
-        if (get_num_active_windows() >= MAX_WINDOWS_NUM) {
+    if (friend_number >= CallControl.max_calls) {
+        return;
+    }
+
+    if (Friends.list[friend_number].window_id == -1) {
+        const int window_id = add_window(toxic, new_chat(toxic->tox, Friends.list[friend_number].num));
+
+        if (window_id < 0) {
+            fprintf(stderr, "Failed to create new chat window in callback_recv_invite()\n");
             return;
         }
 
-        Friends.list[friend_number].chatwin = add_window(toxic, new_chat(toxic->tox, Friends.list[friend_number].num));
+        Friends.list[friend_number].window_id = window_id;
     }
 
     const Call *call = &CallControl.calls[friend_number];
+    Windows *windows = toxic->windows;
 
-    for (uint8_t i = 0; i < MAX_WINDOWS_NUM; ++i) {
-        if (windows[i] != NULL && windows[i]->onInvite != NULL && windows[i]->num == friend_number) {
-            windows[i]->onInvite(windows[i], toxic, friend_number, call->state);
+    for (uint16_t i = 0; i < windows->count; ++i) {
+        ToxWindow *w = windows->list[i];
+
+        if (w->onInvite != NULL && w->num == friend_number) {
+            w->onInvite(w, toxic, friend_number, call->state);
         }
     }
 }
 void callback_recv_ringing(Toxic *toxic, uint32_t friend_number)
 {
-    const Call *call = &CallControl.calls[friend_number];
+    if (friend_number >= CallControl.max_calls) {
+        return;
+    }
 
-    for (uint8_t i = 0; i < MAX_WINDOWS_NUM; ++i) {
-        if (windows[i] != NULL && windows[i]->onRinging != NULL && windows[i]->num == friend_number) {
-            windows[i]->onRinging(windows[i], toxic, friend_number, call->state);
+    const Call *call = &CallControl.calls[friend_number];
+    Windows *windows = toxic->windows;
+
+    for (uint16_t i = 0; i < windows->count; ++i) {
+        ToxWindow *w = windows->list[i];
+
+        if (w->onRinging != NULL && w->num == friend_number) {
+            w->onRinging(w, toxic, friend_number, call->state);
         }
     }
 }
 void callback_recv_starting(Toxic *toxic, uint32_t friend_number)
 {
-    Call *call = &CallControl.calls[friend_number];
+    if (friend_number >= CallControl.max_calls) {
+        return;
+    }
 
-    for (uint8_t i = 0; i < MAX_WINDOWS_NUM; ++i) {
-        if (windows[i] != NULL && windows[i]->onStarting != NULL && windows[i]->num == friend_number) {
-            windows[i]->onStarting(windows[i], toxic, friend_number, call->state);
-            start_call(windows[i], toxic, call);
+    Call *call = &CallControl.calls[friend_number];
+    Windows *windows = toxic->windows;
+
+    for (uint16_t i = 0; i < windows->count; ++i) {
+        ToxWindow *w = windows->list[i];
+
+        if (w->onStarting != NULL && w->num == friend_number) {
+            w->onStarting(w, toxic, friend_number, call->state);
+            start_call(w, toxic, call);
         }
     }
 }
 void callback_call_started(Toxic *toxic, uint32_t friend_number)
 {
+    if (friend_number >= CallControl.max_calls) {
+        return;
+    }
+
     Call *call = &CallControl.calls[friend_number];
+    Windows *windows = toxic->windows;
 
-    for (uint8_t i = 0; i < MAX_WINDOWS_NUM; ++i) {
-        if (windows[i] != NULL && windows[i]->onStart != NULL && windows[i]->num == friend_number) {
-            windows[i]->onStart(windows[i], toxic, friend_number, call->state);
+    for (uint16_t i = 0; i < windows->count; ++i) {
+        ToxWindow *w = windows->list[i];
 
-            start_call(windows[i], toxic, call);
+        if (w->onStart != NULL && w->num == friend_number) {
+            w->onStart(w, toxic, friend_number, call->state);
+            start_call(w, toxic, call);
         }
     }
 }
 void callback_call_canceled(Toxic *toxic, uint32_t friend_number)
 {
-    const Call *call = &CallControl.calls[friend_number];
+    if (friend_number >= CallControl.max_calls) {
+        return;
+    }
 
-    for (uint8_t i = 0; i < MAX_WINDOWS_NUM; ++i) {
-        if (windows[i] != NULL && windows[i]->onCancel != NULL && windows[i]->num == friend_number) {
-            windows[i]->onCancel(windows[i], toxic, friend_number, call->state);
+    const Call *call = &CallControl.calls[friend_number];
+    Windows *windows = toxic->windows;
+
+    for (uint16_t i = 0; i < windows->count; ++i) {
+        ToxWindow *w = windows->list[i];
+
+        if (w->onCancel != NULL && w->num == friend_number) {
+            w->onCancel(w, toxic, friend_number, call->state);
         }
     }
 }
 void callback_call_rejected(Toxic *toxic, uint32_t friend_number)
 {
-    const Call *call = &CallControl.calls[friend_number];
+    if (friend_number >= CallControl.max_calls) {
+        return;
+    }
 
-    for (uint8_t i = 0; i < MAX_WINDOWS_NUM; ++i) {
-        if (windows[i] != NULL && windows[i]->onReject != NULL && windows[i]->num == friend_number) {
-            windows[i]->onReject(windows[i], toxic, friend_number, call->state);
+    const Call *call = &CallControl.calls[friend_number];
+    Windows *windows = toxic->windows;
+
+    for (uint16_t i = 0; i < windows->count; ++i) {
+        ToxWindow *w = windows->list[i];
+
+        if (w->onReject != NULL && w->num == friend_number) {
+            w->onReject(w, toxic, friend_number, call->state);
         }
     }
 }
 void callback_call_ended(Toxic *toxic, uint32_t friend_number)
 {
-    const Call *call = &CallControl.calls[friend_number];
+    if (friend_number >= CallControl.max_calls) {
+        return;
+    }
 
-    for (uint8_t i = 0; i < MAX_WINDOWS_NUM; ++i) {
-        if (windows[i] != NULL && windows[i]->onEnd != NULL && windows[i]->num == friend_number) {
-            windows[i]->onEnd(windows[i], toxic, friend_number, call->state);
+    const Call *call = &CallControl.calls[friend_number];
+    Windows *windows = toxic->windows;
+
+    for (uint16_t i = 0; i < windows->count; ++i) {
+        ToxWindow *w = windows->list[i];
+
+        if (w->onEnd != NULL && w->num == friend_number) {
+            w->onEnd(w, toxic, friend_number, call->state);
         }
     }
 }
@@ -541,6 +588,11 @@ void cmd_call(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc, char (*ar
         return;
     }
 
+    if (self->num >= CallControl.max_calls) {
+        print_err(self, c_config, "Invalid call index");
+        return;
+    }
+
     Call *call = &CallControl.calls[self->num];
 
     if (call->status != cs_None) {
@@ -576,6 +628,11 @@ void cmd_answer(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc, char (*
         return;
     }
 
+    if (self->num >= CallControl.max_calls) {
+        print_err(self, c_config, "Invalid call index");
+        return;
+    }
+
     Call *call = &CallControl.calls[self->num];
 
     if (call->status != cs_Pending) {
@@ -626,6 +683,11 @@ void cmd_reject(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc, char (*
         return;
     }
 
+    if (self->num >= CallControl.max_calls) {
+        print_err(self, c_config, "Invalid call index.");
+        return;
+    }
+
     Call *call = &CallControl.calls[self->num];
 
     if (call->status != cs_Pending) {
@@ -662,6 +724,11 @@ void cmd_hangup(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc, char (*
         return;
     }
 
+    if (self->num >= CallControl.max_calls) {
+        print_err(self, c_config, "Invalid call index.");
+        return;
+    }
+
     Call *call = &CallControl.calls[self->num];
 
     if (call->status == cs_None) {
@@ -796,6 +863,10 @@ void cmd_mute(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc, char (*ar
         return;
     }
 
+    if (self->num >= CallControl.max_calls) {
+        print_err(self, c_config, "Invalid call index.");
+        return;
+    }
 
     /* If call is active, use this_call values */
     Call *this_call = &CallControl.calls[self->num];
@@ -841,6 +912,11 @@ void cmd_sense(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc, char (*a
         return;
     }
 
+    if (self->num >= CallControl.max_calls) {
+        print_err(self, c_config, "Invalid call index.");
+        return;
+    }
+
     const Call *call = &CallControl.calls[self->num];
 
     /* Call must be active */
@@ -862,6 +938,11 @@ void cmd_bitrate(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc, char (
 
     const Client_Config *c_config = toxic->c_config;
 
+    if (self->num >= CallControl.max_calls) {
+        print_err(self, c_config, "Invalid call index.");
+        return;
+    }
+
     Call *call = &CallControl.calls[self->num];
 
     if (call->status != cs_Active) {
@@ -914,13 +995,19 @@ void cmd_bitrate(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc, char (
 
 void place_call(ToxWindow *self, Toxic *toxic)
 {
-    Call *call = &CallControl.calls[self->num];
+    const Client_Config *c_config = toxic->c_config;
 
-    if (call->status != cs_Pending) {
+    if (self->num >= CallControl.max_calls) {
+        print_err(self, c_config, "Invalid call index.");
         return;
     }
 
-    const Client_Config *c_config = toxic->c_config;
+    Call *call = &CallControl.calls[self->num];
+
+    if (call->status != cs_Pending) {
+        print_err(self, toxic->c_config, "No pending call.");
+        return;
+    }
 
     Toxav_Err_Call error;
 
@@ -948,11 +1035,15 @@ void place_call(ToxWindow *self, Toxic *toxic)
 
 void stop_current_call(ToxWindow *self, Toxic *toxic)
 {
+    if (self->num >= CallControl.max_calls) {
+        print_err(self, toxic->c_config, "Invalid call index.");
+        return;
+    }
+
     Call *call = &CallControl.calls[self->num];
 
     if (call->status == cs_Pending) {
         toxav_call_control(toxic->av, self->num, TOXAV_CALL_CONTROL_CANCEL, NULL);
-
         cancel_call(call);
         callback_call_canceled(toxic, self->num);
     } else {
@@ -970,36 +1061,42 @@ void stop_current_call(ToxWindow *self, Toxic *toxic)
 /**
  * Reallocates the Calls list according to n.
  */
-static void realloc_calls(uint32_t n)
+static bool realloc_calls(uint32_t n)
 {
     if (n == 0) {
         free(CallControl.calls);
         CallControl.calls = NULL;
-        return;
+        return true;
     }
 
     Call *temp = realloc(CallControl.calls, n * sizeof(Call));
 
     if (temp == NULL) {
-        exit_toxic_err("failed in realloc_calls", FATALERR_MEMORY);
+        return false;
     }
 
     CallControl.calls = temp;
+    return true;
 }
 
-/**
- * Inits the call structure for a given friend. Called when a friend is added to the friends list.
- * Index must be equivalent to the friend's friendlist index.
- */
-void init_friend_AV(uint32_t index)
+bool init_friend_AV(uint32_t index)
 {
-    if (index == CallControl.max_calls) {
-        realloc_calls(CallControl.max_calls + 1);
-        CallControl.calls[CallControl.max_calls] = (Call) {
-            0
-        };
-        ++CallControl.max_calls;
+    if (index != CallControl.max_calls) {
+        return false;
     }
+
+    if (!realloc_calls(CallControl.max_calls + 1)) {
+        fprintf(stderr, "Warning: realloc_calls(%u) failed\n", CallControl.max_calls + 1);
+        return false;
+    }
+
+    CallControl.calls[CallControl.max_calls] = (Call) {
+        0
+    };
+
+    ++CallControl.max_calls;
+
+    return true;
 }
 
 /**
@@ -1008,7 +1105,11 @@ void init_friend_AV(uint32_t index)
  */
 void del_friend_AV(uint32_t index)
 {
-    realloc_calls(index);
+    if (!realloc_calls(index)) {
+        fprintf(stderr, "Warning: realloc_calls(%u) failed\n", index);
+        return;
+    }
+
     CallControl.max_calls = index;
 }
 
