@@ -1,23 +1,9 @@
 /*  conference.c
  *
+ *  Copyright (C) 2014-2024 Toxic All Rights Reserved.
  *
- *  Copyright (C) 2024 Toxic All Rights Reserved.
- *
- *  This file is part of Toxic.
- *
- *  Toxic is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
- *
- *  Toxic is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with Toxic.  If not, see <http://www.gnu.org/licenses/>.
- *
+ *  This file is part of Toxic. Toxic is free software licensed
+ *  under the GNU General Public License 3.0.
  */
 
 #ifndef _GNU_SOURCE
@@ -62,7 +48,7 @@
 #include "toxic_strings.h"
 #include "windows.h"
 
-#define MAX_CONFERENCE_NUM (MAX_WINDOWS_NUM - 2)
+#define MAX_CONFERENCE_NUM 100
 #define CONFERENCE_EVENT_WAIT 30
 
 static ConferenceChat conferences[MAX_CONFERENCE_NUM];
@@ -73,7 +59,7 @@ extern struct Winthread Winthread;
 static_assert(TOX_CONFERENCE_ID_SIZE == TOX_PUBLIC_KEY_SIZE, "TOX_CONFERENCE_ID_SIZE != TOX_PUBLIC_KEY_SIZE");
 
 /* Array of conference command names used for tab completion. */
-static const char *conference_cmd_list[] = {
+static const char *const conference_cmd_list[] = {
     "/accept",
     "/add",
 #ifdef AUDIO
@@ -142,7 +128,7 @@ void conference_set_title(ToxWindow *self, uint32_t conferencesnum, const char *
     set_window_title(self, title, length);
 }
 
-static void kill_conference_window(ToxWindow *self, const Client_Config *c_config)
+static void kill_conference_window(ToxWindow *self, Windows *windows, const Client_Config *c_config)
 {
     if (self == NULL) {
         return;
@@ -162,7 +148,7 @@ static void kill_conference_window(ToxWindow *self, const Client_Config *c_confi
 
     free(self->help);
     kill_notifs(self->active_box);
-    del_window(self, c_config);
+    del_window(self, windows, c_config);
 }
 
 static void init_conference_logging(ToxWindow *self, Toxic *toxic, uint32_t conferencenum)
@@ -191,13 +177,15 @@ static void init_conference_logging(ToxWindow *self, Toxic *toxic, uint32_t conf
         line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "Failed to load chat history.");
     }
 
-    if (c_config->autolog == AUTOLOG_ON) {
+    if (c_config->autolog) {
         if (log_enable(ctx->log) != 0) {
             line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "Failed to enable chat log.");
         }
     }
 }
 
+static void delete_conference(ToxWindow *self, Toxic *toxic, uint32_t conferencenum);
+
 int init_conference_win(Toxic *toxic, uint32_t conferencenum, uint8_t type, const char *title, size_t length)
 {
     if (toxic == NULL) {
@@ -217,38 +205,49 @@ int init_conference_win(Toxic *toxic, uint32_t conferencenum, uint8_t type, cons
             // probably it so happens that this will (at least typically) be
             // the case, because toxic and tox maintain the indices in
             // parallel ways. But it isn't guaranteed by the API.
-            conferences[i].conferencenum = conferencenum;
-            conferences[i].chatwin = add_window(toxic, self);
+            if (i == max_conference_index) {
+                ++max_conference_index;
+            }
+
             conferences[i].active = true;
+            conferences[i].conferencenum = conferencenum;
             conferences[i].num_peers = 0;
             conferences[i].type = type;
             conferences[i].start_time = get_unix_time();
             conferences[i].audio_enabled = false;
             conferences[i].last_sent_audio = 0;
 
+            const int window_id = add_window(toxic, self);
+
+            if (window_id < 0) {
+                fprintf(stderr, "Failed to create new conference window\n");
+                delete_conference(self, toxic, conferencenum);
+                return -1;
+            }
+
+            conferences[i].window_id = window_id;
+
             if (!tox_conference_get_id(toxic->tox, conferencenum, (uint8_t *) conferences[i].id)) {
                 fprintf(stderr, "Failed to fetch conference ID for conferencenum: %u\n", conferencenum);
+                delete_conference(self, toxic, conferencenum);
+                return -1;
             }
 
 #ifdef AUDIO
             conferences[i].push_to_talk_enabled = toxic->c_config->push_to_talk;
 #endif
 
-            set_active_window_index(conferences[i].chatwin);
+            set_active_window_by_id(toxic->windows, conferences[i].window_id);
 
             conference_set_title(self, conferencenum, title, length);
 
             init_conference_logging(self, toxic, conferencenum);
 
-            if (i == max_conference_index) {
-                ++max_conference_index;
-            }
-
-            return conferences[i].chatwin;
+            return conferences[i].window_id;
         }
     }
 
-    kill_conference_window(self, toxic->c_config);
+    kill_conference_window(self, toxic->windows, toxic->c_config);
 
     return -1;
 }
@@ -264,7 +263,7 @@ static void free_peer(ConferencePeer *peer)
 #endif
 }
 
-void free_conference(ToxWindow *self, const Client_Config *c_config, uint32_t conferencenum)
+void free_conference(ToxWindow *self, Windows *windows, const Client_Config *c_config, uint32_t conferencenum)
 {
     ConferenceChat *chat = &conferences[conferencenum];
 
@@ -299,13 +298,13 @@ void free_conference(ToxWindow *self, const Client_Config *c_config, uint32_t co
     }
 
     max_conference_index = i;
-    kill_conference_window(self, c_config);
+    kill_conference_window(self, windows, c_config);
 }
 
 static void delete_conference(ToxWindow *self, Toxic *toxic, uint32_t conferencenum)
 {
     tox_conference_delete(toxic->tox, conferencenum, NULL);
-    free_conference(self, toxic->c_config, conferencenum);
+    free_conference(self, toxic->windows, toxic->c_config, conferencenum);
 }
 
 void conference_rename_log_path(Toxic *toxic, uint32_t conferencenum, const char *new_title)
@@ -322,8 +321,9 @@ void conference_rename_log_path(Toxic *toxic, uint32_t conferencenum, const char
     char conference_id[TOX_CONFERENCE_ID_SIZE];
     tox_conference_get_id(toxic->tox, conferencenum, (uint8_t *) conference_id);
 
-    if (rename_logfile(toxic->c_config, chat->title, new_title, myid, conference_id, chat->chatwin) != 0) {
-        fprintf(stderr, "Failed to rename conference log to `%s`\n", new_title);
+    if (rename_logfile(toxic->windows, toxic->c_config, chat->title, new_title, myid, conference_id,
+                       chat->window_id) != 0) {
+        fprintf(stderr, "Failed to rename conference log to '%s'\n", new_title);
     }
 }
 
@@ -401,21 +401,23 @@ static void conference_onConferenceMessage(ToxWindow *self, Toxic *toxic, uint32
     /* Only play sound if mentioned by someone else */
     if (strcasestr(msg, selfnick) && strcmp(selfnick, nick)) {
         if (self->active_box != -1) {
-            box_notify2(self, c_config, generic_message, NT_WNDALERT_0 | NT_NOFOCUS | c_config->bell_on_message,
+            box_notify2(self, toxic, generic_message, NT_WNDALERT_0 | NT_NOFOCUS | c_config->bell_on_message,
                         self->active_box, "%s %s", nick, msg);
         } else {
-            box_notify(self, c_config, generic_message, NT_WNDALERT_0 | NT_NOFOCUS | c_config->bell_on_message,
+            box_notify(self, toxic, generic_message, NT_WNDALERT_0 | NT_NOFOCUS | c_config->bell_on_message,
                        &self->active_box, self->name, "%s %s", nick, msg);
         }
 
         nick_clr = RED;
     } else {
-        sound_notify(self, c_config, silent, NT_WNDALERT_1, NULL);
+        sound_notify(self, toxic, silent, NT_WNDALERT_1, NULL);
     }
 
     line_info_add(self, c_config, true, nick, NULL, type == TOX_MESSAGE_TYPE_NORMAL ? IN_MSG : IN_ACTION, 0,
                   nick_clr, "%s", msg);
-    write_to_log(ctx->log, c_config, msg, nick, false);
+
+    write_to_log(ctx->log, c_config, msg, nick,
+                 type == TOX_MESSAGE_TYPE_NORMAL ? LOG_HINT_NORMAL_I : LOG_HINT_ACTION);
 }
 
 static void conference_onConferenceTitleChange(ToxWindow *self, Toxic *toxic, uint32_t conferencenum, uint32_t peernum,
@@ -452,11 +454,12 @@ static void conference_onConferenceTitleChange(ToxWindow *self, Toxic *toxic, ui
 
     char nick[TOX_MAX_NAME_LENGTH];
     get_conference_nick_truncate(tox, nick, peernum, conferencenum);
-    line_info_add(self, c_config, true, nick, NULL, NAME_CHANGE, 0, 0, " set the conference title to: %s", title);
 
     char tmp_event[MAX_STR_SIZE];
-    snprintf(tmp_event, sizeof(tmp_event), "set title to %s", title);
-    write_to_log(ctx->log, c_config, tmp_event, nick, true);
+    snprintf(tmp_event, sizeof(tmp_event), "-!- %s set the conference title to: %s", nick, title);
+
+    line_info_add(self, c_config, true, NULL, NULL, SYS_MSG, true, MAGENTA, "%s", tmp_event);
+    write_to_log(ctx->log, c_config, tmp_event, NULL, LOG_HINT_TOPIC);
 }
 
 /* Puts `(NameListEntry *)`s in `entries` for each matched peer, up to a
@@ -537,7 +540,7 @@ static void conference_update_name_list(uint32_t conferencenum)
     chat->name_list = malloc(chat->num_peers * sizeof(NameListEntry));
 
     if (chat->name_list == NULL) {
-        exit_toxic_err("failed in conference_update_name_list", FATALERR_MEMORY);
+        exit_toxic_err(FATALERR_MEMORY, "failed in conference_update_name_list");
     }
 
     uint32_t count = 0;
@@ -696,8 +699,7 @@ static void update_peer_list(ToxWindow *self, Toxic *toxic, uint32_t conferencen
     ConferencePeer *old_peer_list = malloc(old_num_peers * sizeof(ConferencePeer));
 
     if (!old_peer_list) {
-        exit_toxic_err("failed in update_peer_list", FATALERR_MEMORY);
-        return;
+        exit_toxic_err(FATALERR_MEMORY, "failed in update_peer_list");
     }
 
     if (chat->peer_list != NULL) {
@@ -736,13 +738,16 @@ static void update_peer_list(ToxWindow *self, Toxic *toxic, uint32_t conferencen
 
         size_t length = tox_conference_peer_get_name_size(tox, conferencenum, i, &err);
 
-        if (err != TOX_ERR_CONFERENCE_PEER_QUERY_OK || length >= TOX_MAX_NAME_LENGTH) {
-            // FIXME: length == TOX_MAX_NAME_LENGTH should not be an error!
+        if (err != TOX_ERR_CONFERENCE_PEER_QUERY_OK) {
             continue;
         }
 
+        if (length >= TOX_MAX_NAME_LENGTH) {
+            length = TOX_MAX_NAME_LENGTH - 1;
+        }
+
         tox_conference_peer_get_name(tox, conferencenum, i, (uint8_t *) peer->name, &err);
-        peer->name[length] = 0;
+        peer->name[length] = '\0';
 
         if (err != TOX_ERR_CONFERENCE_PEER_QUERY_OK) {
             continue;
@@ -754,8 +759,8 @@ static void update_peer_list(ToxWindow *self, Toxic *toxic, uint32_t conferencen
 
         if (new_peer && peer->name_length > 0 && timed_out(chat->start_time, CONFERENCE_EVENT_WAIT)) {
             const char *msg = "has joined the conference";
-            line_info_add(self, c_config, true, peer->name, NULL, CONNECTION, 0, GREEN, msg);
-            write_to_log(ctx->log, c_config, msg, peer->name, true);
+            line_info_add(self, c_config, true, peer->name, NULL, CONNECTION, 0, GREEN, "%s", msg);
+            write_to_log(ctx->log, c_config, msg, peer->name, LOG_HINT_CONNECT);
         }
 
 #ifdef AUDIO
@@ -771,8 +776,8 @@ static void update_peer_list(ToxWindow *self, Toxic *toxic, uint32_t conferencen
         if (old_peer->active) {
             if (old_peer->name_length > 0 && !find_peer_by_pubkey(chat->peer_list, chat->num_peers, old_peer->pubkey, NULL)) {
                 const char *msg = "has left the conference";
-                line_info_add(self, c_config, true, old_peer->name, NULL, DISCONNECTION, 0, RED, msg);
-                write_to_log(ctx->log, c_config, msg, old_peer->name, true);
+                line_info_add(self, c_config, true, old_peer->name, NULL, DISCONNECTION, 0, RED, "%s", msg);
+                write_to_log(ctx->log, c_config, msg, old_peer->name, LOG_HINT_DISCONNECT);
             }
 
             free_peer(old_peer);
@@ -842,17 +847,18 @@ static void conference_onConferencePeerNameChange(ToxWindow *self, Toxic *toxic,
         ChatContext *ctx = self->chatwin;
 
         if (peer->name_length > 0) {
-            char log_event[TOXIC_MAX_NAME_LENGTH * 2 + 32];
-            line_info_add(self, c_config, true, peer->name, (const char *) name, NAME_CHANGE, 0, 0, " is now known as ");
+            char log_event[TOX_MAX_NAME_LENGTH * 2 + 32];
+            line_info_add(self, c_config, true, peer->name, (const char *) name, NAME_CHANGE, 0, 0,
+                          " is now known as ");
 
             snprintf(log_event, sizeof(log_event), "is now known as %s", (const char *) name);
-            write_to_log(ctx->log, c_config, log_event, peer->name, true);
+            write_to_log(ctx->log, c_config, log_event, peer->name, LOG_HINT_NAME);
 
             // this is kind of a hack; peers always join a group with no name set and then set it after
         } else if (timed_out(conferences[conferencenum].start_time, CONFERENCE_EVENT_WAIT)) {
             const char *msg = "has joined the conference";
-            line_info_add(self, c_config, true, name, NULL, CONNECTION, 0, GREEN, msg);
-            write_to_log(ctx->log, c_config, msg, name, true);
+            line_info_add(self, c_config, true, name, NULL, CONNECTION, 0, GREEN, "%s", msg);
+            write_to_log(ctx->log, c_config, msg, name, LOG_HINT_CONNECT);
         }
     }
 
@@ -915,7 +921,7 @@ static bool conference_onKey(ToxWindow *self, Toxic *toxic, wint_t key, bool ltr
     }
 
     if (ltr || key == L'\n') {    /* char is printable */
-        input_new_char(self, c_config, key, x, x2);
+        input_new_char(self, toxic, key, x, x2);
         return true;
     }
 
@@ -923,7 +929,7 @@ static bool conference_onKey(ToxWindow *self, Toxic *toxic, wint_t key, bool ltr
         return true;
     }
 
-    if (input_handle(self, c_config, key, x, x2)) {
+    if (input_handle(self, toxic, key, x, x2)) {
         return true;
     }
 
@@ -998,10 +1004,10 @@ static bool conference_onKey(ToxWindow *self, Toxic *toxic, wint_t key, bool ltr
                     ctx->start = wlen < x2 ? 0 : wlen - x2 + 1;
                 }
             } else {
-                sound_notify(self, c_config, notif_error, 0, NULL);
+                sound_notify(self, toxic, notif_error, 0, NULL);
             }
         } else {
-            sound_notify(self, c_config, notif_error, 0, NULL);
+            sound_notify(self, toxic, notif_error, 0, NULL);
         }
     } else if (key == T_KEY_C_DOWN) {    /* Scroll peerlist up and down one position */
         input_ret = true;
@@ -1214,7 +1220,6 @@ static void conference_onDraw(ToxWindow *self, Toxic *toxic)
         mvwaddch(ctx->sidebar, line, 0, ACS_LTEE);
         mvwhline(ctx->sidebar, line, 1, ACS_HLINE, SIDEBAR_WIDTH - 1);
         wattroff(ctx->sidebar, COLOR_PAIR(PEERLIST_LINE));
-        ++line;
 
         for (uint32_t i = 0;
                 i < num_peers && i < y2 - header_lines - CHATBOX_HEIGHT;
@@ -1232,7 +1237,7 @@ static void conference_onDraw(ToxWindow *self, Toxic *toxic)
     int new_x = ctx->start ? x2 - 1 : MAX(0, wcswidth(ctx->line, ctx->pos));
     wmove(self->window, y, new_x);
 
-    draw_window_bar(self);
+    draw_window_bar(self, toxic->windows);
 
     wnoutrefresh(self->window);
 
@@ -1251,7 +1256,7 @@ static void conference_onInit(ToxWindow *self, Toxic *toxic)
     getmaxyx(self->window, y2, x2);
 
     if (x2 <= 0 || y2 <= 0) {
-        exit_toxic_err("failed in conference_onInit", FATALERR_CURSES);
+        exit_toxic_err(FATALERR_CURSES, "failed in conference_onInit");
     }
 
     ChatContext *ctx = self->chatwin;
@@ -1265,7 +1270,7 @@ static void conference_onInit(ToxWindow *self, Toxic *toxic)
     ctx->log = calloc(1, sizeof(struct chatlog));
 
     if (ctx->log == NULL || ctx->hst == NULL) {
-        exit_toxic_err("failed in conference_onInit", FATALERR_MEMORY);
+        exit_toxic_err(FATALERR_MEMORY, "failed in conference_onInit");
     }
 
     line_info_init(ctx->hst);
@@ -1308,7 +1313,7 @@ static int get_conferencenum_by_public_key_string(const char *public_key)
  *
  * Return false if conference does not exist.
  */
-static bool conference_window_set_tab_name_colour(const char *public_key, int colour)
+static bool conference_window_set_tab_name_colour(Windows *windows, const char *public_key, int colour)
 {
     const int conferencenum = get_conferencenum_by_public_key_string(public_key);
 
@@ -1316,7 +1321,7 @@ static bool conference_window_set_tab_name_colour(const char *public_key, int co
         return false;
     }
 
-    ToxWindow *self = get_window_by_number_type(conferencenum, WINDOW_TYPE_CONFERENCE);
+    ToxWindow *self = get_window_by_number_type(windows, conferencenum, WINDOW_TYPE_CONFERENCE);
 
     if (self == NULL) {
         return false;
@@ -1327,7 +1332,7 @@ static bool conference_window_set_tab_name_colour(const char *public_key, int co
     return true;
 }
 
-bool conference_config_set_tab_name_colour(const char *public_key, const char *colour)
+bool conference_config_set_tab_name_colour(Windows *windows, const char *public_key, const char *colour)
 {
     const int colour_val = colour_string_to_int(colour);
 
@@ -1335,10 +1340,10 @@ bool conference_config_set_tab_name_colour(const char *public_key, const char *c
         return false;
     }
 
-    return conference_window_set_tab_name_colour(public_key, colour_val);
+    return conference_window_set_tab_name_colour(windows, public_key, colour_val);
 }
 
-bool conference_config_set_autolog(const char *public_key, bool autolog_enabled)
+bool conference_config_set_autolog(Windows *windows, const char *public_key, bool autolog_enabled)
 {
     const int conferencenum = get_conferencenum_by_public_key_string(public_key);
 
@@ -1347,8 +1352,8 @@ bool conference_config_set_autolog(const char *public_key, bool autolog_enabled)
     }
 
     return autolog_enabled
-           ? enable_window_log_by_number_type(conferencenum, WINDOW_TYPE_CONFERENCE)
-           : disable_window_log_by_number_type(conferencenum, WINDOW_TYPE_CONFERENCE);
+           ? enable_window_log_by_number_type(windows, conferencenum, WINDOW_TYPE_CONFERENCE)
+           : disable_window_log_by_number_type(windows, conferencenum, WINDOW_TYPE_CONFERENCE);
 }
 
 static ToxWindow *new_conference_chat(uint32_t conferencenum)
@@ -1356,7 +1361,7 @@ static ToxWindow *new_conference_chat(uint32_t conferencenum)
     ToxWindow *ret = calloc(1, sizeof(ToxWindow));
 
     if (ret == NULL) {
-        exit_toxic_err("failed in new_conference_chat", FATALERR_MEMORY);
+        exit_toxic_err(FATALERR_MEMORY, "failed in new_conference_chat");
     }
 
     ret->type = WINDOW_TYPE_CONFERENCE;
@@ -1375,7 +1380,7 @@ static ToxWindow *new_conference_chat(uint32_t conferencenum)
     Help *help = calloc(1, sizeof(Help));
 
     if (chatwin == NULL || help == NULL) {
-        exit_toxic_err("failed in new_conference_chat", FATALERR_MEMORY);
+        exit_toxic_err(FATALERR_MEMORY, "failed in new_conference_chat");
     }
 
     ret->chatwin = chatwin;
