@@ -1,23 +1,9 @@
 /*  conference.h
  *
+ *  Copyright (C) 2014-2024 Toxic All Rights Reserved.
  *
- *  Copyright (C) 2024 Toxic All Rights Reserved.
- *
- *  This file is part of Toxic.
- *
- *  Toxic is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
- *
- *  Toxic is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with Toxic.  If not, see <http://www.gnu.org/licenses/>.
- *
+ *  This file is part of Toxic. Toxic is free software licensed
+ *  under the GNU General Public License 3.0.
  */
 
 #ifndef CONFERENCE_H
@@ -61,7 +47,7 @@ typedef struct NameListEntry {
 
 typedef struct {
     uint32_t conferencenum;
-    int chatwin;
+    uint16_t window_id;
     bool active;
     uint8_t type;
     int side_pos;    /* current position of the sidebar - used for scrolling up and down */
@@ -88,7 +74,7 @@ typedef struct {
 } ConferenceChat;
 
 /* Frees all Toxic associated data structures for a conference (does not call tox_conference_delete() ) */
-void free_conference(ToxWindow *self, const Client_Config *c_config, uint32_t conferencenum);
+void free_conference(ToxWindow *self, Windows *windows, const Client_Config *c_config, uint32_t conferencenum);
 
 int init_conference_win(Toxic *toxic, uint32_t conferencenum, uint8_t type, const char *title, size_t length);
 
@@ -115,7 +101,7 @@ uint32_t get_name_list_entries_by_prefix(uint32_t conferencenum, const char *pre
  *
  * Return true on success.
  */
-bool conference_config_set_tab_name_colour(const char *public_key, const char *colour);
+bool conference_config_set_tab_name_colour(Windows *windows, const char *public_key, const char *colour);
 
 /*
  * Sets the auto-logging preference for the conference associated with `public_key`.
@@ -124,7 +110,7 @@ bool conference_config_set_tab_name_colour(const char *public_key, const char *c
  *
  * Return true on success.
  */
-bool conference_config_set_autolog(const char *public_key, bool autolog_enabled);
+bool conference_config_set_autolog(Windows *windows, const char *public_key, bool autolog_enabled);
 
 /* Enable audio in a conference.
  *
