@@ -1,23 +1,9 @@
 /*  execute.c
  *
+ *  Copyright (C) 2014-2024 Toxic All Rights Reserved.
  *
- *  Copyright (C) 2024 Toxic All Rights Reserved.
- *
- *  This file is part of Toxic.
- *
- *  Toxic is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
- *
- *  Toxic is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with Toxic.  If not, see <http://www.gnu.org/licenses/>.
- *
+ *  This file is part of Toxic. Toxic is free software licensed
+ *  under the GNU General Public License 3.0.
  */
 
 #include <stdlib.h>
@@ -36,7 +22,7 @@
 #include "windows.h"
 
 struct cmd_func {
-    const char *name;
+    const char *const name;
     void (*func)(WINDOW *w, ToxWindow *, Toxic *toxic, int argc, char (*argv)[MAX_STR_SIZE]);
 };
 
@@ -224,7 +210,7 @@ static int parse_command(const char *input, char (*args)[MAX_STR_SIZE])
     char *cmd = strdup(input);
 
     if (cmd == NULL) {
-        exit_toxic_err("failed in parse_command", FATALERR_MEMORY);
+        return -1;
     }
 
     int num_args = 0;
@@ -273,9 +259,10 @@ void execute(WINDOW *w, ToxWindow *self, Toxic *toxic, const char *input, int mo
     }
 
     char args[MAX_NUM_ARGS][MAX_STR_SIZE];
-    int num_args = parse_command(input, args);
+    const int num_args = parse_command(input, args);
 
     if (num_args <= 0) {
+        fprintf(stderr, "Warning: parse_command(%s, %d) failed in execute()\n", input, mode);
         return;
     }
 
