@@ -1,23 +1,9 @@
 /*  game_base.c
  *
+ *  Copyright (C) 2020-2024 Toxic All Rights Reserved.
  *
- *  Copyright (C) 2024 Toxic All Rights Reserved.
- *
- *  This file is part of Toxic.
- *
- *  Toxic is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
- *
- *  Toxic is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with Toxic.  If not, see <http://www.gnu.org/licenses/>.
- *
+ *  This file is part of Toxic. Toxic is free software licensed
+ *  under the GNU General Public License 3.0.
  */
 
 #include <stdlib.h>
@@ -148,21 +134,22 @@ static bool game_type_is_multi_and_single(const ToxWindow *window, GameType type
  */
 void game_window_notify(const GameData *game, const char *message)
 {
-    ToxWindow *self = get_window_ptr(game->window_id);
+    const Toxic *toxic = game->toxic;
+    const Client_Config *c_config = toxic->c_config;
+
+    ToxWindow *self = get_window_pointer_by_id(toxic->windows, game->window_id);
 
     if (self == NULL) {
         return;
     }
 
-    const Client_Config *c_config = game->toxic->c_config;
-
     const int bell_on_message = c_config->bell_on_message;
 
     if (self->active_box != -1) {
-        box_notify2(self, c_config, generic_message, NT_WNDALERT_0 | NT_NOFOCUS | bell_on_message,
+        box_notify2(self, toxic, generic_message, NT_WNDALERT_0 | NT_NOFOCUS | bell_on_message,
                     self->active_box, "%s", message);
     } else {
-        box_notify(self, c_config, generic_message, NT_WNDALERT_0 | NT_NOFOCUS | bell_on_message,
+        box_notify(self, toxic, generic_message, NT_WNDALERT_0 | NT_NOFOCUS | bell_on_message,
                    &self->active_box, self->name, "%s", message);
     }
 }
@@ -175,7 +162,7 @@ TIME_MS get_time_millis(void)
     return ((TIME_MS) t.tv_sec) * 1000 + ((TIME_MS) t.tv_nsec) / 1000000;
 }
 
-void game_kill(ToxWindow *self, const Client_Config *c_config)
+void game_kill(ToxWindow *self, Windows *windows, const Client_Config *c_config)
 {
     GameData *game = self->game;
 
@@ -190,17 +177,18 @@ void game_kill(ToxWindow *self, const Client_Config *c_config)
     }
 
     kill_notifs(self->active_box);
-    del_window(self, c_config);
+    del_window(self, windows, c_config);
 
-    if (get_num_active_windows_type(WINDOW_TYPE_GAME) == 0) {
+    if (get_num_active_windows_type(windows, WINDOW_TYPE_GAME) == 0) {
         set_window_refresh_rate(NCURSES_DEFAULT_REFRESH_RATE);
     }
 }
 
-static void game_init_abort(const ToxWindow *parent, ToxWindow *self, const Client_Config *c_config)
+static void game_init_abort(const ToxWindow *parent, ToxWindow *self, Windows *windows,
+                            const Client_Config *c_config)
 {
-    game_kill(self, c_config);
-    set_active_window_index(parent->index);
+    game_kill(self, windows, c_config);
+    set_active_window_by_id(windows, parent->id);
 }
 
 static void game_toggle_pause(GameData *game)
@@ -274,7 +262,7 @@ int game_initialize(const ToxWindow *parent, Toxic *toxic, GameType type, uint32
 
     const int window_id = add_window(toxic, self);
 
-    if (window_id == -1) {
+    if (window_id < 0) {
         free(game);
         free(self);
         return -4;
@@ -284,12 +272,12 @@ int game_initialize(const ToxWindow *parent, Toxic *toxic, GameType type, uint32
 
     if (game->is_multiplayer) {
         if (parent->type != WINDOW_TYPE_CHAT) {
-            game_init_abort(parent, self, c_config);
+            game_init_abort(parent, self, toxic->windows, c_config);
             return -3;
         }
 
         if (get_friend_connection_status(parent->num) == TOX_CONNECTION_NONE) {
-            game_init_abort(parent, self, c_config);
+            game_init_abort(parent, self, toxic->windows, c_config);
             return -2;
         }
 
@@ -308,20 +296,20 @@ int game_initialize(const ToxWindow *parent, Toxic *toxic, GameType type, uint32
     game->friend_number = parent->num;
 
     if (game->window == NULL) {
-        game_init_abort(parent, self, c_config);
+        game_init_abort(parent, self, toxic->windows, c_config);
         return -4;
     }
 
     const int init_ret = game_initialize_type(game, multiplayer_data, length, self_host);
 
     if (init_ret < 0) {
-        game_init_abort(parent, self, c_config);
+        game_init_abort(parent, self, toxic->windows, c_config);
         return init_ret;
     }
 
     game->status = GS_Running;
 
-    set_active_window_index(window_id);
+    set_active_window_by_id(toxic->windows, game->window_id);
 
     set_window_refresh_rate(NCURSES_GAME_REFRESH_RATE);
 
@@ -743,7 +731,7 @@ static void game_onDraw(ToxWindow *self, Toxic *toxic)
     GameData *game = self->game;
 
     game_draw_help_bar(game, self->window);
-    draw_window_bar(self);
+    draw_window_bar(self, toxic->windows);
 
     curs_set(0);
 
@@ -795,7 +783,7 @@ static bool game_onKey(ToxWindow *self, Toxic *toxic, wint_t key, bool is_printa
 
     if (key == KEY_F(9)) {
         pthread_mutex_lock(&Winthread.lock);
-        game_kill(self, toxic->c_config);
+        game_kill(self, toxic->windows, toxic->c_config);
         pthread_mutex_unlock(&Winthread.lock);
         return true;
     }
@@ -841,7 +829,7 @@ static void game_onInit(ToxWindow *self, Toxic *toxic)
     getmaxyx(self->window, max_y, max_x);
 
     if (max_y <= 0 || max_x <= 0) {
-        exit_toxic_err("failed in game_onInit", FATALERR_CURSES);
+        exit_toxic_err(FATALERR_CURSES, "failed in game_onInit");
     }
 
     self->window_bar = subwin(self->window, WINDOW_BAR_HEIGHT, max_x, max_y - 2, 0);
@@ -930,11 +918,11 @@ static ToxWindow *game_new_window(Tox *tox, GameType type, uint32_t friendnumber
     ret->active_box = -1;
 
     if (game_type_is_multi_only(type)) {
-        char nick[TOX_MAX_NAME_LENGTH];
-        get_nick_truncate(tox, nick, friendnumber);
+        char name[TOXIC_MAX_NAME_LENGTH + 1];
+        get_friend_name(name, sizeof(name), friendnumber);
 
-        char buf[sizeof(nick) + sizeof(ret->name) + 4];
-        snprintf(buf, sizeof(buf), "%s (%s)", window_name, nick);
+        char buf[sizeof(name) + sizeof(ret->name) + 4];
+        snprintf(buf, sizeof(buf), "%s (%s)", window_name, name);
 
         const size_t name_size = sizeof(ret->name);
 
