@@ -1,23 +1,9 @@
 /*  game_base.h
  *
+ *  Copyright (C) 2020-2024 Toxic All Rights Reserved.
  *
- *  Copyright (C) 2024 Toxic All Rights Reserved.
- *
- *  This file is part of Toxic.
- *
- *  Toxic is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
- *
- *  Toxic is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with Toxic.  If not, see <http://www.gnu.org/licenses/>.
- *
+ *  This file is part of Toxic. Toxic is free software licensed
+ *  under the GNU General Public License 3.0.
  */
 
 #ifndef GAME_BASE
@@ -151,7 +137,7 @@ struct GameData {
     int        parent_max_x; // max dimensions of parent window
     int        parent_max_y;
 
-    int        window_id;
+    uint16_t   window_id;
     WINDOW     *window;
 
     Toxic      *toxic;  // must be locked with Winthread mutex
@@ -394,7 +380,7 @@ TIME_MS get_time_millis(void);
 /*
  * Ends game associated with `self` and cleans up.
  */
-void game_kill(ToxWindow *self, const Client_Config *c_config);
+void game_kill(ToxWindow *self, Windows *windows, const Client_Config *c_config);
 
 /*
  * Sends a packet containing payload `data` of size `length` to the friendnumber associated with the game's
