@@ -1,29 +1,16 @@
 /*  line_info.c
  *
+ *  Copyright (C) 2014-2024 Toxic All Rights Reserved.
  *
- *  Copyright (C) 2024 Toxic All Rights Reserved.
- *
- *  This file is part of Toxic.
- *
- *  Toxic is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
- *
- *  Toxic is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with Toxic.  If not, see <http://www.gnu.org/licenses/>.
- *
+ *  This file is part of Toxic. Toxic is free software licensed
+ *  under the GNU General Public License 3.0.
  */
 
 #ifndef _GNU_SOURCE
 #define _GNU_SOURCE    /* needed for wcswidth() */
 #endif
 
+#include <errno.h>
 #include <stdarg.h>
 #include <stdbool.h>
 #include <stdlib.h>
@@ -44,7 +31,7 @@ void line_info_init(struct history *hst)
     hst->line_root = calloc(1, sizeof(struct line_info));
 
     if (hst->line_root == NULL) {
-        exit_toxic_err("failed in line_info_init", FATALERR_MEMORY);
+        exit_toxic_err(FATALERR_MEMORY, "calloc() failed in line_info_init()");
     }
 
     hst->line_start = hst->line_root;
@@ -66,7 +53,7 @@ void line_info_reset_start(ToxWindow *self, struct history *hst)
     getmaxyx(self->window, y2, x2);
     UNUSED_VAR(x2);
 
-    int top_offst = (self->type == WINDOW_TYPE_CHAT) || (self->type == WINDOW_TYPE_PROMPT) ? TOP_BAR_HEIGHT : 0;
+    int top_offst = self->type != WINDOW_TYPE_CONFERENCE ? TOP_BAR_HEIGHT : 0;
     int max_y = y2 - CHATBOX_HEIGHT - WINDOW_BAR_HEIGHT - top_offst;
 
     uint16_t curlines = 0;
@@ -348,7 +335,7 @@ static int print_wrap(WINDOW *win, struct line_info *line, int max_x, int max_y)
  *
  * Returns the widechar width of the string.
  */
-static uint16_t line_info_add_msg(wchar_t *buf, size_t buf_size, const char *msg)
+uint16_t line_info_add_msg(wchar_t *buf, size_t buf_size, const char *msg)
 {
     if (msg == NULL || msg[0] == '\0') {
         return 0;
@@ -366,7 +353,8 @@ static uint16_t line_info_add_msg(wchar_t *buf, size_t buf_size, const char *msg
 
         return (uint16_t)width;
     } else {
-        fprintf(stderr, "Failed to convert string '%s' to widechar\n", msg);
+        fprintf(stderr, "Failed to convert string '%s' to widechar (len=%d, error=%s)\n",
+                msg, wc_msg_len, strerror(errno));
         const wchar_t *err = L"Failed to parse message";
         uint16_t width = (uint16_t)wcslen(err);
         wmemcpy(buf, err, width);
@@ -389,39 +377,12 @@ static void line_info_init_line(ToxWindow *self, struct line_info *line)
     print_wrap(NULL, line, max_x, max_y);
 }
 
-/* creates new line_info line and puts it in the queue.
- *
- * Returns the id of the new line.
- * Returns -1 on failure.
+/*
+ * Returns the base format length required for a given line type (e.g. spaces, newlines etc.)
  */
-int line_info_add(ToxWindow *self, const Client_Config *c_config, bool show_timestamp, const char *name1,
-                  const char *name2, LINE_TYPE type, uint8_t bold, uint8_t colour, const char *msg, ...)
+static uint16_t line_info_type_length(const Client_Config *c_config, LINE_TYPE type)
 {
-    if (self == NULL) {
-        return -1;
-    }
-
-    struct history *hst = self->chatwin->hst;
-
-    if (hst->queue_size >= MAX_LINE_INFO_QUEUE) {
-        return -1;
-    }
-
-    struct line_info *new_line = calloc(1, sizeof(struct line_info));
-
-    if (new_line == NULL) {
-        exit_toxic_err("failed in line_info_add", FATALERR_MEMORY);
-    }
-
-    char frmt_msg[MAX_LINE_INFO_MSG_SIZE];
-    frmt_msg[0] = 0;
-
-    va_list args;
-    va_start(args, msg);
-    vsnprintf(frmt_msg, sizeof(frmt_msg), msg, args);
-    va_end(args);
-
-    int len = 1;     /* there will always be a newline */
+    uint16_t len = 1;  // there will always be a newline
 
     /* for type-specific formatting in print function */
     switch (type) {
@@ -474,11 +435,48 @@ int line_info_add(ToxWindow *self, const Client_Config *c_config, bool show_time
             break;
     }
 
+    return len;
+}
+
+/* creates new line_info line and puts it in the queue.
+ *
+ * Returns the id of the new line.
+ * Returns -1 on failure.
+ */
+int line_info_add(ToxWindow *self, const Client_Config *c_config, bool show_timestamp, const char *name1,
+                  const char *name2, LINE_TYPE type, uint8_t bold, uint8_t colour, const char *msg, ...)
+{
+    if (self == NULL) {
+        return -1;
+    }
+
+    struct history *hst = self->chatwin->hst;
+
+    if (hst->queue_size >= MAX_LINE_INFO_QUEUE) {
+        return -1;
+    }
+
+    struct line_info *new_line = calloc(1, sizeof(struct line_info));
+
+    if (new_line == NULL) {
+        exit_toxic_err(FATALERR_MEMORY, "failed in line_info_add");
+    }
+
+    char frmt_msg[MAX_LINE_INFO_MSG_SIZE];
+    frmt_msg[0] = 0;
+
+    va_list args;
+    va_start(args, msg);
+    vsnprintf(frmt_msg, sizeof(frmt_msg), msg, args);
+    va_end(args);
+
+    int len = line_info_type_length(c_config, type);
+
     const uint16_t msg_width = line_info_add_msg(new_line->msg, sizeof(new_line->msg) / sizeof(wchar_t), frmt_msg);
     len += msg_width;
 
     if (show_timestamp)  {
-        if (c_config->timestamps == TIMESTAMPS_ON) {
+        if (c_config->show_timestamps) {
             get_time_str(new_line->timestr, sizeof(new_line->timestr), c_config->timestamp_format);
         }
 
@@ -510,7 +508,60 @@ int line_info_add(ToxWindow *self, const Client_Config *c_config, bool show_time
 
     line_info_init_line(self, new_line);
 
-    hst->queue[hst->queue_size++] = new_line;
+    hst->queue[hst->queue_size] = new_line;
+    ++hst->queue_size;
+
+    return new_line->id;
+}
+
+int line_info_load_history(ToxWindow *self, const Client_Config *c_config, const char *timestamp,
+                           const char *name, LINE_TYPE type, bool bold, int colour, const char *message)
+{
+    if (self == NULL) {
+        return -1;
+    }
+
+    struct history *hst = self->chatwin->hst;
+
+    if (hst->queue_size >= MAX_LINE_INFO_QUEUE) {
+        return -1;
+    }
+
+    struct line_info *new_line = calloc(1, sizeof(struct line_info));
+
+    if (new_line == NULL) {
+        return -1;
+    }
+
+    int len = line_info_type_length(c_config, type);
+
+    const uint16_t msg_width = line_info_add_msg(new_line->msg, sizeof(new_line->msg) / sizeof(wchar_t), message);
+    len += msg_width;
+
+    if (c_config->show_timestamps) {
+        snprintf(new_line->timestr, sizeof(new_line->timestr), "%s", timestamp);
+    }
+
+    len += strlen(new_line->timestr) + 1;
+
+    if (name != NULL) {
+        snprintf(new_line->name1, sizeof(new_line->name1), "%s", name);
+        len += strlen(new_line->name1);
+    }
+
+    new_line->id = (hst->line_end->id + 1 + hst->queue_size) % INT_MAX;
+    new_line->len = len;
+    new_line->msg_width = msg_width;
+    new_line->type = type;
+    new_line->bold = bold;
+    new_line->colour = colour;
+    new_line->noread_flag = false;
+    new_line->timestamp = get_unix_time();
+
+    line_info_init_line(self, new_line);
+
+    hst->queue[hst->queue_size] = new_line;
+    ++hst->queue_size;
 
     return new_line->id;
 }
@@ -566,7 +617,7 @@ void line_info_print(ToxWindow *self, const Client_Config *c_config)
         return;
     }
 
-    if (self->type == WINDOW_TYPE_CONFERENCE || self->type == WINDOW_TYPE_GROUPCHAT) {
+    if (self->type == WINDOW_TYPE_CONFERENCE) {
         wmove(win, 0, 0);
     } else {
         wmove(win, TOP_BAR_HEIGHT, 0);
