@@ -1,25 +1,12 @@
 /*  misc_tools.c
  *
+ *  Copyright (C) 2014-2024 Toxic All Rights Reserved.
  *
- *  Copyright (C) 2024 Toxic All Rights Reserved.
- *
- *  This file is part of Toxic.
- *
- *  Toxic is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
- *
- *  Toxic is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with Toxic.  If not, see <http://www.gnu.org/licenses/>.
- *
+ *  This file is part of Toxic. Toxic is free software licensed
+ *  under the GNU General Public License 3.0.
  */
 
+#include <assert.h>
 #include <arpa/inet.h>
 #include <ctype.h>
 #include <limits.h>
@@ -29,6 +16,7 @@
 #include <time.h>
 
 #include "file_transfers.h"
+#include "friendlist.h"
 #include "misc_tools.h"
 #include "settings.h"
 #include "toxic.h"
@@ -99,51 +87,51 @@ void get_time_str(char *buf, size_t bufsize, const char *format_string)
         return;
     }
 
-    if (strftime(buf, bufsize, format_string, get_time()) > 0) {
+    if (format_time_str(buf, bufsize, format_string, get_time()) > 0) {
         return;
     }
 
-    if (strftime(buf, bufsize, TIMESTAMP_DEFAULT, get_time()) > 0) {
+    if (format_time_str(buf, bufsize, TIMESTAMP_DEFAULT, get_time()) > 0) {
         return;
     }
 
     buf[0] = '\0';
 }
 
-/* Converts seconds to string in format HH:mm:ss; truncates hours and minutes when necessary */
-void get_elapsed_time_str(char *buf, int bufsize, time_t secs)
+void get_elapsed_time_str(char *buf, int bufsize, uint64_t elapsed_seconds)
 {
-    if (!secs) {
+    if (elapsed_seconds == 0) {
+        snprintf(buf, bufsize, "<Invalid time format>");
         return;
     }
 
-    long int seconds = secs % 60;
-    long int minutes = (secs % 3600) / 60;
-    long int hours = secs / 3600;
+    long int seconds = elapsed_seconds % 60;
+    long int minutes = (elapsed_seconds % 3600) / 60;
+    long int hours = elapsed_seconds / 3600;
 
-    if (!minutes && !hours) {
+    if (minutes == 0 && hours == 0) {
         snprintf(buf, bufsize, "%.2ld", seconds);
-    } else if (!hours) {
+    } else if (hours == 0) {
         snprintf(buf, bufsize, "%ld:%.2ld", minutes, seconds);
     } else {
         snprintf(buf, bufsize, "%ld:%.2ld:%.2ld", hours, minutes, seconds);
     }
 }
 
-/* Converts seconds to string in format H hours, m minutes, s seconds */
-void get_elapsed_time_str_alt(char *buf, int bufsize, uint64_t secs)
+void get_elapsed_time_str_alt(char *buf, int bufsize, uint64_t elapsed_seconds)
 {
-    if (!secs) {
+    if (elapsed_seconds == 0) {
+        snprintf(buf, bufsize, "<Invalid time format>");
         return;
     }
 
-    long int seconds = secs % 60;
-    long int minutes = (secs % 3600) / 60;
-    long int hours = secs / 3600;
+    long int seconds = elapsed_seconds % 60;
+    long int minutes = (elapsed_seconds % 3600) / 60;
+    long int hours = elapsed_seconds / 3600;
 
-    if (!minutes && !hours) {
+    if (minutes == 0 && hours == 0) {
         snprintf(buf, bufsize, "%ld seconds", seconds);
-    } else if (!hours) {
+    } else if (hours == 0) {
         snprintf(buf, bufsize, "%ld minutes, %ld seconds", minutes, seconds);
     } else {
         snprintf(buf, bufsize, "%ld hours, %ld minutes, %ld seconds", hours, minutes, seconds);
@@ -303,19 +291,19 @@ int qsort_ptr_char_array_helper(const void *str1, const void *str2)
     return strcasecmp(*(const char *const *)str1, *(const char *const *)str2);
 }
 
-static const char invalid_chars[] = {'/', '\n', '\t', '\v', '\r', '\0'};
+/* List of characters we don't allow in nicks. */
+static const char invalid_nick_chars[] = {':', '/', '\0', '\a', '\b', '\f', '\n', '\r', '\t', '\v'};
+
+/* List of characters we don't allow in single-line strings. */
+static const char invalid_string_chars[] = {'\0', '\a', '\b', '\f', '\n', '\r', '\t', '\v'};
 
 /*
- * Helper function for `valid_nick()`.
- *
- * Returns true if `ch` is not in the `invalid_chars` array.
+ * Returns true if the character `ch` is not in the supplied `invalid_chars` array.
  */
-static bool is_valid_char(char ch)
+static bool is_valid_char(char ch, const char *const invalid_chars, size_t list_length)
 {
-    char tmp;
-
-    for (size_t i = 0; (tmp = invalid_chars[i]); ++i) {
-        if (tmp == ch) {
+    for (size_t i = 0; i < list_length; ++i) {
+        if (invalid_chars[i] == ch) {
             return false;
         }
     }
@@ -323,25 +311,17 @@ static bool is_valid_char(char ch)
     return true;
 }
 
-/* Returns true if nick is valid.
- *
- * A valid toxic nick:
- * - cannot be empty
- * - cannot start with a space
- * - must not contain a forward slash (for logfile naming purposes)
- * - must not contain contiguous spaces
- * - must not contain a newline or tab seqeunce
- */
 bool valid_nick(const char *nick)
 {
     if (!nick[0] || nick[0] == ' ') {
         return false;
     }
 
-    for (size_t i = 0; nick[i]; ++i) {
+    for (size_t i = 0; nick[i] != '\0'; ++i) {
         char ch = nick[i];
 
-        if ((ch == ' ' && nick[i + 1] == ' ') || !is_valid_char(ch)) {
+        if ((ch == ' ' && nick[i + 1] == ' ')
+                || !is_valid_char(ch, invalid_nick_chars, sizeof(invalid_nick_chars))) {
             return false;
         }
     }
@@ -349,38 +329,39 @@ bool valid_nick(const char *nick)
     return true;
 }
 
-/* Converts all newline/tab chars to spaces (use for strings that should be contained to a single line) */
-void filter_str(char *str, size_t len)
+void filter_string(char *str, size_t len, bool is_nick)
 {
+    const char *const invalid_chars = is_nick ? invalid_nick_chars : invalid_string_chars;
+    const size_t list_length = is_nick ? sizeof(invalid_nick_chars) : sizeof(invalid_string_chars);
+
     for (size_t i = 0; i < len; ++i) {
         char ch = str[i];
 
-        if (!is_valid_char(ch) || str[i] == '\0') {
+        if (!is_valid_char(ch, invalid_chars, list_length)) {
             str[i] = ' ';
         }
     }
 }
 
-/* gets base file name from path or original file name if no path is supplied.
- * Returns the file name length
- */
-size_t get_file_name(char *namebuf, size_t bufsize, const char *pathname)
+int get_file_name(char *namebuf, size_t bufsize, const char *pathname)
 {
     int len = strlen(pathname) - 1;
     char *path = strdup(pathname);
 
     if (path == NULL) {
-        exit_toxic_err("failed in get_file_name", FATALERR_MEMORY);
+        return -1;
     }
 
     while (len >= 0 && pathname[len] == '/') {
-        path[len--] = '\0';
+        path[len] = '\0';
+        --len;
     }
 
     char *finalname = strdup(path);
 
     if (finalname == NULL) {
-        exit_toxic_err("failed in get_file_name", FATALERR_MEMORY);
+        free(path);
+        return -1;
     }
 
     const char *basenm = strrchr(path, '/');
@@ -395,7 +376,7 @@ size_t get_file_name(char *namebuf, size_t bufsize, const char *pathname)
     free(finalname);
     free(path);
 
-    return strlen(namebuf);
+    return (int)strlen(namebuf);
 }
 
 /* Gets the base directory of path and puts it in dir.
@@ -431,11 +412,10 @@ void str_to_lower(char *str)
     }
 }
 
-/* puts friendnum's nick in buf, truncating at TOXIC_MAX_NAME_LENGTH if necessary.
-   if toxcore API call fails, put UNKNOWN_NAME in buf
-   Returns nick len */
-size_t get_nick_truncate(Tox *tox, char *buf, uint32_t friendnum)
+size_t get_nick_truncate(Tox *tox, char *buf, uint16_t buf_size, uint32_t friendnum)
 {
+    assert(buf_size > 0);
+
     Tox_Err_Friend_Query err;
     size_t len = tox_friend_get_name_size(tox, friendnum, &err);
 
@@ -447,16 +427,15 @@ size_t get_nick_truncate(Tox *tox, char *buf, uint32_t friendnum)
         }
     }
 
-    len = MIN(len, TOXIC_MAX_NAME_LENGTH - 1);
+    len = MIN(len, buf_size - 1);
     buf[len] = '\0';
-    filter_str(buf, len);
+    filter_string(buf, len, false);
+
     return len;
 
 on_error:
-    strcpy(buf, UNKNOWN_NAME);
-    len = strlen(UNKNOWN_NAME);
-    buf[len] = '\0';
-    return len;
+    snprintf(buf, buf_size, "%s", UNKNOWN_NAME);
+    return strlen(buf);
 }
 
 /* same as get_nick_truncate but for conferences */
@@ -475,7 +454,7 @@ int get_conference_nick_truncate(Tox *tox, char *buf, uint32_t peernum, uint32_t
 
     len = MIN(len, TOXIC_MAX_NAME_LENGTH - 1);
     buf[len] = '\0';
-    filter_str(buf, len);
+    filter_string(buf, len, true);
     return len;
 
 on_error:
@@ -507,7 +486,7 @@ size_t get_group_nick_truncate(Tox *tox, char *buf, uint32_t peer_id, uint32_t g
     len = MIN(len, TOXIC_MAX_NAME_LENGTH - 1);
     buf[len] = '\0';
 
-    filter_str(buf, len);
+    filter_string(buf, len, true);
 
     return len;
 }
@@ -531,22 +510,26 @@ size_t get_group_self_nick_truncate(Tox *tox, char *buf, uint32_t groupnum)
     }
 
     len = MIN(len, TOXIC_MAX_NAME_LENGTH - 1);
-    buf[len] = 0;
+    buf[len] = '\0';
 
-    filter_str(buf, len);
+    filter_string(buf, len, true);
 
     return len;
 }
 
-/* copies data to msg buffer, removing return characters.
-   returns length of msg, which will be no larger than size-1 */
 size_t copy_tox_str(char *msg, size_t size, const char *data, size_t length)
 {
     size_t j = 0;
 
     for (size_t i = 0; (i < length) && (j < size - 1); ++i) {
-        if (data[i] != '\r') {
-            msg[j++] = data[i];
+        const char ch = data[i];
+
+        if (ch == '\t' || ch == '\v') {
+            msg[j] = ' ';
+            ++j;
+        } else if (ch != '\r') {
+            msg[j] = ch;
+            ++j;
         }
     }
 
@@ -563,9 +546,9 @@ int char_find(int idx, const char *s, char ch)
         return 0;
     }
 
-    int i = idx;
+    int i;
 
-    for (i = idx; s[i]; ++i) {
+    for (i = idx; s[i] != '\0'; ++i) {
         if (s[i] == ch) {
             break;
         }
@@ -798,3 +781,11 @@ int colour_string_to_int(const char *colour)
 
     return -1;
 }
+
+size_t format_time_str(char *s, size_t max, const char *format, const struct tm *tm)
+{
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wformat-nonliteral"
+    return strftime(s, max, format, tm);
+#pragma GCC diagnostic pop
+}
