@@ -1,25 +1,12 @@
 /*  notify.c
  *
+ *  Copyright (C) 2014-2024 Toxic All Rights Reserved.
  *
- *  Copyright (C) 2024 Toxic All Rights Reserved.
- *
- *  This file is part of Toxic.
- *
- *  Toxic is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
- *
- *  Toxic is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with Toxic.  If not, see <http://www.gnu.org/licenses/>.
- *
+ *  This file is part of Toxic. Toxic is free software licensed
+ *  under the GNU General Public License 3.0.
  */
 
+#include <assert.h>
 #include <stdarg.h>
 #include <stdbool.h>
 #include <stdio.h>
@@ -34,7 +21,10 @@
 #include "misc_tools.h"
 #include "notify.h"
 #include "settings.h"
+
+#ifdef X11
 #include "x11focus.h"
+#endif  /* X11 */
 
 #if defined(AUDIO) || defined(SOUND_NOTIFY)
 #ifdef __APPLE__
@@ -55,12 +45,20 @@
 
 #ifdef BOX_NOTIFY
 #include <libnotify/notify.h>
-#endif
+#endif  /* BOX_NOTIFY */
 
 #define MAX_BOX_MSG_LEN 127
-#define SOUNDS_SIZE 10
 #define ACTIVE_NOTIFS_MAX 10
 
+#ifdef SOUND_NOTIFY
+#define SOUNDS_SIZE 10
+#endif  /* SOUND_NOTIFY */
+
+#define CONTENT_HIDDEN_MESSAGE "[Content hidden]"
+
+static_assert(sizeof(CONTENT_HIDDEN_MESSAGE) < MAX_BOX_MSG_LEN,
+              "sizeof(CONTENT_HIDDEN_MESSAGE) >= MAX_BOX_MSG_LEN");
+
 static struct Control {
     time_t cooldown;
     time_t notif_timeout;
@@ -127,15 +125,17 @@ static void tab_notify(ToxWindow *self, uint64_t flags)
     ++self->pending_messages;
 }
 
-static bool notifications_are_disabled(const Client_Config *c_config, uint64_t flags)
+static bool notifications_are_disabled(const Toxic *toxic, uint64_t flags)
 {
-    if (c_config->alerts != ALERTS_ENABLED) {
+    const Client_Config *c_config = toxic->c_config;
+
+    if (!c_config->alerts) {
         return true;
     }
 
     bool res = (flags & NT_RESTOL) && (Control.cooldown > get_unix_time());
 #ifdef X11
-    return res || ((flags & NT_NOFOCUS) && is_focused());
+    return res || ((flags & NT_NOFOCUS) && is_focused(&toxic->x11_focus));
 #else
     return res;
 #endif
@@ -167,7 +167,7 @@ static bool is_playing(int source)
 /* cooldown is in seconds */
 #define DEVICE_COOLDOWN 5 /* TODO perhaps load this from config? */
 static bool device_opened = false;
-time_t last_opened_update = 0;
+static time_t last_opened_update = 0;
 
 /* Opens primary device. Returns true on succe*/
 static void m_open_device(const Client_Config *c_config)
@@ -348,7 +348,7 @@ static int play_source(uint32_t source, uint32_t buffer, bool looping)
 }
 
 #elif BOX_NOTIFY
-void *do_playing(void *_p)
+static void *do_playing(void *_p)
 {
     UNUSED_VAR(_p);
 
@@ -375,7 +375,7 @@ void *do_playing(void *_p)
     pthread_exit(NULL);
 }
 
-void graceful_clear(void)
+static void graceful_clear(void)
 {
     control_lock();
 
@@ -598,12 +598,14 @@ static int m_play_sound(const Client_Config *c_config, Notification notif, uint6
 #endif /* SOUND_NOTIFY */
 }
 
-int sound_notify(ToxWindow *self, const Client_Config *c_config, Notification notif, uint64_t flags,
+int sound_notify(ToxWindow *self, const Toxic *toxic, Notification notif, uint64_t flags,
                  int *id_indicator)
 {
+    const Client_Config *c_config = toxic->c_config;
+
     tab_notify(self, flags);
 
-    if (notifications_are_disabled(c_config, flags)) {
+    if (notifications_are_disabled(toxic, flags)) {
         return -1;
     }
 
@@ -639,11 +641,11 @@ int sound_notify(ToxWindow *self, const Client_Config *c_config, Notification no
     return id;
 }
 
-int sound_notify2(ToxWindow *self, const Client_Config *c_config, Notification notif, uint64_t flags, int id)
+int sound_notify2(ToxWindow *self, const Toxic *toxic, Notification notif, uint64_t flags, int id)
 {
     tab_notify(self, flags);
 
-    if (notifications_are_disabled(c_config, flags)) {
+    if (notifications_are_disabled(toxic, flags)) {
         return -1;
     }
 
@@ -659,7 +661,7 @@ int sound_notify2(ToxWindow *self, const Client_Config *c_config, Notification n
         return -1;
     }
 
-    m_open_device(c_config);
+    m_open_device(toxic->c_config);
 
     alSourceStop(actives[id].source);
     alDeleteSources(1, &actives[id].source);
@@ -686,17 +688,19 @@ int sound_notify2(ToxWindow *self, const Client_Config *c_config, Notification n
 #endif /* SOUND_NOTIFY */
 }
 
-int box_notify(ToxWindow *self, const Client_Config *c_config, Notification notif, uint64_t flags,
+int box_notify(ToxWindow *self, const Toxic *toxic, Notification notif, uint64_t flags,
                int *id_indicator, const char *title, const char *format, ...)
 {
-    if (notifications_are_disabled(c_config, flags)) {
+    if (notifications_are_disabled(toxic, flags)) {
         tab_notify(self, flags);
         return -1;
     }
 
 #ifdef BOX_NOTIFY
 
-    int id = sound_notify(self, c_config, notif, flags, id_indicator);
+    const Client_Config *c_config = toxic->c_config;
+
+    int id = sound_notify(self, toxic, notif, flags, id_indicator);
 
     control_lock();
 
@@ -734,10 +738,14 @@ int box_notify(ToxWindow *self, const Client_Config *c_config, Notification noti
         strcpy(actives[id].title + 20, "...");
     }
 
-    va_list __ARGS__;
-    va_start(__ARGS__, format);
-    vsnprintf(actives[id].messages[0], MAX_BOX_MSG_LEN, format, __ARGS__);
-    va_end(__ARGS__);
+    if (c_config->show_notification_content) {
+        va_list __ARGS__;
+        va_start(__ARGS__, format);
+        vsnprintf(actives[id].messages[0], MAX_BOX_MSG_LEN, format, __ARGS__);
+        va_end(__ARGS__);
+    } else {
+        snprintf(actives[id].messages[0], MAX_BOX_MSG_LEN, "%s", CONTENT_HIDDEN_MESSAGE);
+    }
 
     if (strlen(actives[id].messages[0]) > MAX_BOX_MSG_LEN - 3) {
         strcpy(actives[id].messages[0] + MAX_BOX_MSG_LEN - 3, "...");
@@ -749,27 +757,28 @@ int box_notify(ToxWindow *self, const Client_Config *c_config, Notification noti
 
     notify_notification_set_timeout(actives[id].box, Control.notif_timeout);
     notify_notification_set_app_name(actives[id].box, "toxic");
-    /*notify_notification_add_action(actives[id].box, "lel", "default", m_notify_action, self, NULL);*/
     notify_notification_show(actives[id].box, NULL);
 
     control_unlock();
     return id;
 #else
-    return sound_notify(self, c_config, notif, flags, id_indicator);
+    return sound_notify(self, toxic, notif, flags, id_indicator);
 #endif /* BOX_NOTIFY */
 }
 
-int box_notify2(ToxWindow *self, const Client_Config *c_config, Notification notif, uint64_t flags,
+int box_notify2(ToxWindow *self, const Toxic *toxic, Notification notif, uint64_t flags,
                 int id, const char *format, ...)
 {
-    if (notifications_are_disabled(c_config, flags)) {
+    if (notifications_are_disabled(toxic, flags)) {
         tab_notify(self, flags);
         return -1;
     }
 
 #ifdef BOX_NOTIFY
 
-    if (sound_notify2(self, c_config, notif, flags, id) == -1) {
+    const Client_Config *c_config = toxic->c_config;
+
+    if (sound_notify2(self, toxic, notif, flags, id) == -1) {
         return -1;
     }
 
@@ -780,10 +789,14 @@ int box_notify2(ToxWindow *self, const Client_Config *c_config, Notification not
         return -1;
     }
 
-    va_list __ARGS__;
-    va_start(__ARGS__, format);
-    vsnprintf(actives[id].messages[actives[id].size], MAX_BOX_MSG_LEN, format, __ARGS__);
-    va_end(__ARGS__);
+    if (c_config->show_notification_content) {
+        va_list __ARGS__;
+        va_start(__ARGS__, format);
+        vsnprintf(actives[id].messages[actives[id].size], MAX_BOX_MSG_LEN, format, __ARGS__);
+        va_end(__ARGS__);
+    } else {
+        snprintf(actives[id].messages[actives[id].size], MAX_BOX_MSG_LEN, "%s", CONTENT_HIDDEN_MESSAGE);
+    }
 
     if (strlen(actives[id].messages[actives[id].size]) > MAX_BOX_MSG_LEN - 3) {
         strcpy(actives[id].messages[actives[id].size] + MAX_BOX_MSG_LEN - 3, "...");
@@ -808,21 +821,23 @@ int box_notify2(ToxWindow *self, const Client_Config *c_config, Notification not
 
     return id;
 #else
-    return sound_notify2(self, c_config, notif, flags, id);
+    return sound_notify2(self, toxic, notif, flags, id);
 #endif /* BOX_NOTIFY */
 }
 
-int box_silent_notify(ToxWindow *self, const Client_Config *c_config, uint64_t flags, int *id_indicator,
+int box_silent_notify(ToxWindow *self, const Toxic *toxic, uint64_t flags, int *id_indicator,
                       const char *title, const char *format, ...)
 {
     tab_notify(self, flags);
 
-    if (notifications_are_disabled(c_config, flags)) {
+    if (notifications_are_disabled(toxic, flags)) {
         return -1;
     }
 
 #ifdef BOX_NOTIFY
 
+    const Client_Config *c_config = toxic->c_config;
+
     control_lock();
 
     int id;
@@ -845,10 +860,14 @@ int box_silent_notify(ToxWindow *self, const Client_Config *c_config, uint64_t f
         strcpy(actives[id].title + 20, "...");
     }
 
-    va_list __ARGS__;
-    va_start(__ARGS__, format);
-    vsnprintf(actives[id].messages[0], MAX_BOX_MSG_LEN, format, __ARGS__);
-    va_end(__ARGS__);
+    if (c_config->show_notification_content) {
+        va_list __ARGS__;
+        va_start(__ARGS__, format);
+        vsnprintf(actives[id].messages[0], MAX_BOX_MSG_LEN, format, __ARGS__);
+        va_end(__ARGS__);
+    } else {
+        snprintf(actives[id].messages[0], MAX_BOX_MSG_LEN, "%s", CONTENT_HIDDEN_MESSAGE);
+    }
 
     if (strlen(actives[id].messages[0]) > MAX_BOX_MSG_LEN - 3) {
         strcpy(actives[id].messages[0] + MAX_BOX_MSG_LEN - 3, "...");
@@ -861,7 +880,6 @@ int box_silent_notify(ToxWindow *self, const Client_Config *c_config, uint64_t f
 
     notify_notification_set_timeout(actives[id].box, Control.notif_timeout);
     notify_notification_set_app_name(actives[id].box, "toxic");
-    /*notify_notification_add_action(actives[id].box, "lel", "default", m_notify_action, self, NULL);*/
     notify_notification_show(actives[id].box, NULL);
 
     control_unlock();
@@ -871,16 +889,19 @@ int box_silent_notify(ToxWindow *self, const Client_Config *c_config, uint64_t f
 #endif /* BOX_NOTIFY */
 }
 
-int box_silent_notify2(ToxWindow *self, const Client_Config *c_config, uint64_t flags, int id,
+int box_silent_notify2(ToxWindow *self, const Toxic *toxic, uint64_t flags, int id,
                        const char *format, ...)
 {
     tab_notify(self, flags);
 
-    if (notifications_are_disabled(c_config, flags)) {
+    if (notifications_are_disabled(toxic, flags)) {
         return -1;
     }
 
 #ifdef BOX_NOTIFY
+
+    const Client_Config *c_config = toxic->c_config;
+
     control_lock();
 
     if (id < 0 || id >= ACTIVE_NOTIFS_MAX || !actives[id].box || actives[id].size >= MAX_BOX_MSG_LEN + 1) {
@@ -888,11 +909,14 @@ int box_silent_notify2(ToxWindow *self, const Client_Config *c_config, uint64_t
         return -1;
     }
 
-
-    va_list __ARGS__;
-    va_start(__ARGS__, format);
-    vsnprintf(actives[id].messages[actives[id].size], MAX_BOX_MSG_LEN, format, __ARGS__);
-    va_end(__ARGS__);
+    if (c_config->show_notification_content) {
+        va_list __ARGS__;
+        va_start(__ARGS__, format);
+        vsnprintf(actives[id].messages[actives[id].size], MAX_BOX_MSG_LEN, format, __ARGS__);
+        va_end(__ARGS__);
+    } else {
+        snprintf(actives[id].messages[actives[id].size], MAX_BOX_MSG_LEN, "%s", CONTENT_HIDDEN_MESSAGE);
+    }
 
     if (strlen(actives[id].messages[actives[id].size]) > MAX_BOX_MSG_LEN - 3) {
         strcpy(actives[id].messages[actives[id].size] + MAX_BOX_MSG_LEN - 3, "...");
