@@ -1,23 +1,9 @@
 /*  prompt.c
  *
+ *  Copyright (C) 2014-2024 Toxic All Rights Reserved.
  *
- *  Copyright (C) 2024 Toxic All Rights Reserved.
- *
- *  This file is part of Toxic.
- *
- *  Toxic is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
- *
- *  Toxic is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with Toxic.  If not, see <http://www.gnu.org/licenses/>.
- *
+ *  This file is part of Toxic. Toxic is free software licensed
+ *  under the GNU General Public License 3.0.
  */
 
 #ifndef _GNU_SOURCE
@@ -49,7 +35,7 @@ extern FriendsList Friends;
 FriendRequests FrndRequests;
 
 /* Array of global command names used for tab completion. */
-static const char *glob_cmd_list[] = {
+static const char *const glob_cmd_list[] = {
     "/accept",
     "/add",
     "/avatar",
@@ -99,7 +85,7 @@ static const char *glob_cmd_list[] = {
 
 };
 
-void kill_prompt_window(ToxWindow *self, const Client_Config *c_config)
+void kill_prompt_window(ToxWindow *self, Windows *windows, const Client_Config *c_config)
 {
     ChatContext *ctx = self->chatwin;
     StatusBar *statusbar = self->stb;
@@ -118,7 +104,7 @@ void kill_prompt_window(ToxWindow *self, const Client_Config *c_config)
     free(self->help);
     free(statusbar);
 
-    del_window(self, c_config);
+    del_window(self, windows, c_config);
 }
 
 /* callback: Updates own connection status in prompt statusbar */
@@ -239,7 +225,7 @@ static bool prompt_onKey(ToxWindow *self, Toxic *toxic, wint_t key, bool ltr)
     }
 
     if (ltr || key == '\n') {    /* char is printable */
-        input_new_char(self, c_config, key, x, x2);
+        input_new_char(self, toxic, key, x, x2);
         return true;
     }
 
@@ -247,7 +233,7 @@ static bool prompt_onKey(ToxWindow *self, Toxic *toxic, wint_t key, bool ltr)
         return true;
     }
 
-    if (input_handle(self, c_config, key, x, x2)) {
+    if (input_handle(self, toxic, key, x, x2)) {
         return true;
     }
 
@@ -257,7 +243,7 @@ static bool prompt_onKey(ToxWindow *self, Toxic *toxic, wint_t key, bool ltr)
         input_ret = true;
 
         if (ctx->len > 1 && ctx->line[0] == '/') {
-            int diff = -1;
+            int diff;
 
             if (wcsncmp(ctx->line, L"/avatar ", wcslen(L"/avatar ")) == 0) {
                 diff = dir_match(self, toxic, ctx->line, L"/avatar");
@@ -280,10 +266,10 @@ static bool prompt_onKey(ToxWindow *self, Toxic *toxic, wint_t key, bool ltr)
                     ctx->start = wlen < x2 ? 0 : wlen - x2 + 1;
                 }
             } else {
-                sound_notify(self, c_config, notif_error, 0, NULL);
+                sound_notify(self, toxic, notif_error, 0, NULL);
             }
         } else {
-            sound_notify(self, c_config, notif_error, 0, NULL);
+            sound_notify(self, toxic, notif_error, 0, NULL);
         }
     } else if (key == '\r') {
         input_ret = true;
@@ -451,13 +437,13 @@ static void prompt_onDraw(ToxWindow *self, Toxic *toxic)
     self->x = x2;
 
     /* Truncate note if it doesn't fit in statusbar */
-    uint16_t maxlen = x2 - getcurx(statusbar->topline) - 3;
+    const int maxlen = x2 - getcurx(statusbar->topline) - 3;
 
     pthread_mutex_lock(&Winthread.lock);
-    size_t statusmsg_len = statusbar->statusmsg_len;
+    const size_t statusmsg_len = statusbar->statusmsg_len;
     pthread_mutex_unlock(&Winthread.lock);
 
-    if (statusmsg_len > maxlen) {
+    if (statusmsg_len > maxlen && maxlen >= 3) {
         pthread_mutex_lock(&Winthread.lock);
         statusbar->statusmsg[maxlen - 3] = 0;
         strcat(statusbar->statusmsg, "...");
@@ -483,10 +469,10 @@ static void prompt_onDraw(ToxWindow *self, Toxic *toxic)
 
     UNUSED_VAR(x);
 
-    int new_x = ctx->start ? x2 - 1 : MAX(0, wcswidth(ctx->line, ctx->pos));
+    const int new_x = ctx->start ? x2 - 1 : MAX(0, wcswidth(ctx->line, ctx->pos));
     wmove(self->window, y, new_x);
 
-    draw_window_bar(self);
+    draw_window_bar(self, toxic->windows);
 
     wnoutrefresh(self->window);
 
@@ -505,41 +491,37 @@ static void prompt_onConnectionChange(ToxWindow *self, Toxic *toxic, uint32_t fr
     const Client_Config *c_config = toxic->c_config;
     ChatContext *ctx = self->chatwin;
 
-    char nick[TOX_MAX_NAME_LENGTH] = {0};    /* stop removing this initiation */
-    get_nick_truncate(toxic->tox, nick, friendnum);
-
-    if (!nick[0]) {
-        snprintf(nick, sizeof(nick), "%s", UNKNOWN_NAME);
-    }
-
-    const char *msg;
-
     if (!friend_config_get_show_connection_msg(friendnum)) {
         return;
     }
 
+    char nick[TOXIC_MAX_NAME_LENGTH + 1];
+    get_friend_name(nick, sizeof(nick), friendnum);
+
+    const char *msg;
+
     if (connection_status != TOX_CONNECTION_NONE && Friends.list[friendnum].connection_status == TOX_CONNECTION_NONE) {
         msg = "has come online";
-        line_info_add(self, c_config, true, nick, NULL, CONNECTION, 0, GREEN, msg);
-        write_to_log(ctx->log, c_config, msg, nick, true);
+        line_info_add(self, c_config, true, nick, NULL, CONNECTION, 0, GREEN, "%s", msg);
+        write_to_log(ctx->log, c_config, msg, nick, LOG_HINT_CONNECT);
 
         if (self->active_box != -1) {
-            box_notify2(self, c_config, user_log_in, NT_WNDALERT_2 | NT_NOTIFWND | NT_RESTOL, self->active_box,
+            box_notify2(self, toxic, user_log_in, NT_WNDALERT_2 | NT_NOTIFWND | NT_RESTOL, self->active_box,
                         "%s has come online", nick);
         } else {
-            box_notify(self, c_config, user_log_in, NT_WNDALERT_2 | NT_NOTIFWND | NT_RESTOL, &self->active_box,
+            box_notify(self, toxic, user_log_in, NT_WNDALERT_2 | NT_NOTIFWND | NT_RESTOL, &self->active_box,
                        "Toxic", "%s has come online", nick);
         }
     } else if (connection_status == TOX_CONNECTION_NONE) {
         msg = "has gone offline";
-        line_info_add(self, c_config, true, nick, NULL, DISCONNECTION, 0, RED, msg);
-        write_to_log(ctx->log, c_config, msg, nick, true);
+        line_info_add(self, c_config, true, nick, NULL, DISCONNECTION, 0, RED, "%s", msg);
+        write_to_log(ctx->log, c_config, msg, nick, LOG_HINT_DISCONNECT);
 
         if (self->active_box != -1) {
-            box_notify2(self, c_config, user_log_out, NT_WNDALERT_2 | NT_NOTIFWND | NT_RESTOL, self->active_box,
+            box_notify2(self, toxic, user_log_out, NT_WNDALERT_2 | NT_NOTIFWND | NT_RESTOL, self->active_box,
                         "%s has gone offline", nick);
         } else {
-            box_notify(self, c_config, user_log_out, NT_WNDALERT_2 | NT_NOTIFWND | NT_RESTOL, &self->active_box,
+            box_notify(self, toxic, user_log_out, NT_WNDALERT_2 | NT_NOTIFWND | NT_RESTOL, &self->active_box,
                        "Toxic", "%s has gone offline", nick);
         }
     }
@@ -574,10 +556,8 @@ static void prompt_onFriendRequest(ToxWindow *self, Toxic *toxic, const char *ke
     }
 
     const Client_Config *c_config = toxic->c_config;
-    ChatContext *ctx = self->chatwin;
 
     line_info_add(self, c_config, true, NULL, NULL, SYS_MSG, 0, 0, "Friend request with the message '%s'", data);
-    write_to_log(ctx->log, c_config, "Friend request with the message '%s'", "", true);
 
     if (key_is_similar(key)) {
         line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, RED,
@@ -589,13 +569,12 @@ static void prompt_onFriendRequest(ToxWindow *self, Toxic *toxic, const char *ke
     const int n = add_friend_request(key, data);
 
     if (n == -1) {
-        const char *errmsg = "Friend request queue is full. Discarding request.";
-        line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, errmsg);
+        line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "Friend request queue is full. Discarding request.");
         return;
     }
 
     line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "Type \"/accept %d\" or \"/decline %d\"", n, n);
-    sound_notify(self, c_config, generic_message, NT_WNDALERT_1 | NT_NOTIFWND, NULL);
+    sound_notify(self, toxic, generic_message, NT_WNDALERT_1 | NT_NOTIFWND, NULL);
 }
 
 void prompt_init_statusbar(Toxic *toxic, bool first_time_run)
@@ -603,14 +582,14 @@ void prompt_init_statusbar(Toxic *toxic, bool first_time_run)
     ToxWindow *self = toxic->home_window;
 
     if (self == NULL) {
-        exit_toxic_err("failed in prompt_init_statusbar", FATALERR_WININIT);
+        exit_toxic_err(FATALERR_WININIT, "failed in prompt_init_statusbar");
     }
 
     int x2, y2;
     getmaxyx(self->window, y2, x2);
 
     if (y2 <= 0 || x2 <= 0) {
-        exit_toxic_err("failed in prompt_init_statusbar", FATALERR_CURSES);
+        exit_toxic_err(FATALERR_CURSES, "failed in prompt_init_statusbar");
     }
 
     UNUSED_VAR(y2);
@@ -622,10 +601,10 @@ void prompt_init_statusbar(Toxic *toxic, bool first_time_run)
     statusbar->status = TOX_USER_STATUS_NONE;
     statusbar->connection = TOX_CONNECTION_NONE;
 
-    char nick[TOX_MAX_NAME_LENGTH];
-    char statusmsg[TOX_MAX_STATUS_MESSAGE_LENGTH];
+    char nick[TOX_MAX_NAME_LENGTH + 1];
+    char statusmsg[TOX_MAX_STATUS_MESSAGE_LENGTH + 1];
 
-    size_t n_len = tox_self_get_name_size(tox);
+    const size_t n_len = tox_self_get_name_size(tox);
     tox_self_get_name(tox, (uint8_t *) nick);
 
     size_t s_len = tox_self_get_status_message_size(tox);
@@ -657,14 +636,11 @@ static void print_welcome_msg(ToxWindow *self, const Client_Config *c_config)
     line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 1, BLUE, "     | || | | \\  / | | |    ");
     line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 1, BLUE, "     | || |_| /  \\ | | |___ ");
     line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 1, BLUE,
-                  "     |_| \\___/_/\\_\\___\\____| v." TOXICVER);
-    line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "");
-
-    const char *msg = "Welcome to Toxic, a free, open source Tox-based instant messaging client.";
-    line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 1, CYAN, msg);
-    msg = "Type \"/help\" for assistance. Further help may be found via the man page.";
-    line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 1, CYAN, msg);
-    line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "");
+                  "     |_| \\___/_/\\_\\___\\____| v.%s\n", TOXICVER);
+    line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 1, CYAN,
+                  "Welcome to Toxic, a free, open source Tox-based instant messaging client.");
+    line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 1, CYAN,
+                  "Type \"/help\" for assistance. Further help may be found via the man page.\n");
 }
 
 static void prompt_init_log(Toxic *toxic)
@@ -681,7 +657,7 @@ static void prompt_init_log(Toxic *toxic)
         return;
     }
 
-    if (toxic->c_config->autolog == AUTOLOG_ON) {
+    if (toxic->c_config->autolog) {
         if (log_enable(ctx->log) == -1) {
             line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "Warning: Failed to enable log.");
         }
@@ -701,7 +677,7 @@ static void prompt_onInit(ToxWindow *self, Toxic *toxic)
     getmaxyx(self->window, y2, x2);
 
     if (y2 <= 0 || x2 <= 0) {
-        exit_toxic_err("failed in prompt_onInit", FATALERR_CURSES);
+        exit_toxic_err(FATALERR_CURSES, "failed in prompt_onInit");
     }
 
     ChatContext *ctx = self->chatwin;
@@ -714,7 +690,7 @@ static void prompt_onInit(ToxWindow *self, Toxic *toxic)
     ctx->hst = calloc(1, sizeof(struct history));
 
     if (ctx->log == NULL || ctx->hst == NULL) {
-        exit_toxic_err("failed in prompt_onInit", FATALERR_MEMORY);
+        exit_toxic_err(FATALERR_MEMORY, "failed in prompt_onInit");
     }
 
     line_info_init(ctx->hst);
@@ -724,7 +700,7 @@ static void prompt_onInit(ToxWindow *self, Toxic *toxic)
     scrollok(ctx->history, 0);
     wmove(self->window, y2 - CURS_Y_OFFSET, 0);
 
-    if (toxic->c_config->show_welcome_msg == SHOW_WELCOME_MSG_ON) {
+    if (toxic->c_config->show_welcome_msg) {
         print_welcome_msg(toxic->home_window, toxic->c_config);
     }
 }
@@ -734,7 +710,7 @@ ToxWindow *new_prompt(void)
     ToxWindow *ret = calloc(1, sizeof(ToxWindow));
 
     if (ret == NULL) {
-        exit_toxic_err("failed in new_prompt", FATALERR_MEMORY);
+        exit_toxic_err(FATALERR_MEMORY, "failed in new_prompt");
     }
 
     ret->num = -1;
@@ -753,7 +729,7 @@ ToxWindow *new_prompt(void)
     Help *help = calloc(1, sizeof(Help));
 
     if (stb == NULL || chatwin == NULL || help == NULL) {
-        exit_toxic_err("failed in new_prompt", FATALERR_MEMORY);
+        exit_toxic_err(FATALERR_MEMORY, "failed in new_prompt");
     }
 
     ret->chatwin = chatwin;
