@@ -1,23 +1,9 @@
 /*  settings.c
  *
+ *  Copyright (C) 2014-2024 Toxic All Rights Reserved.
  *
- *  Copyright (C) 2024 Toxic All Rights Reserved.
- *
- *  This file is part of Toxic.
- *
- *  Toxic is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
- *
- *  Toxic is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with Toxic.  If not, see <http://www.gnu.org/licenses/>.
- *
+ *  This file is part of Toxic. Toxic is free software licensed
+ *  under the GNU General Public License 3.0.
  */
 
 #include <assert.h>
@@ -51,7 +37,9 @@
 
 static_assert(MAX_STR_SIZE > TOXIC_CONF_FILE_EXT_LENGTH, "MAX_STR_SIZE <= TOXIC_CONF_FILE_EXT_LENGTH");
 
+#ifdef SOUND_NOTIFY
 #define NO_SOUND "silent"
+#endif  /* SOUND_NOTIFY */
 
 static struct ui_strings {
     const char *self;
@@ -60,6 +48,7 @@ static struct ui_strings {
     const char *timestamp_format;
     const char *log_timestamp_format;
     const char *alerts;
+    const char *show_notification_content;
     const char *bell_on_message;
     const char *bell_on_filetrans;
     const char *bell_on_filetrans_accept;
@@ -97,6 +86,7 @@ static struct ui_strings {
     "timestamp_format",
     "log_timestamp_format",
     "alerts",
+    "show_notification_content",
     "bell_on_message",
     "bell_on_filetrans",
     "bell_on_filetrans_accept",
@@ -130,24 +120,24 @@ static struct ui_strings {
 
 static void ui_defaults(Client_Config *settings)
 {
-    settings->timestamps = TIMESTAMPS_ON;
     snprintf(settings->timestamp_format, sizeof(settings->timestamp_format), "%s", TIMESTAMP_DEFAULT);
     snprintf(settings->log_timestamp_format, sizeof(settings->log_timestamp_format), "%s", LOG_TIMESTAMP_DEFAULT);
-
-    settings->autolog = AUTOLOG_OFF;
-    settings->alerts = ALERTS_ENABLED;
+    settings->show_timestamps = true;
+    settings->autolog = false;
+    settings->alerts = true;
+    settings->show_notification_content = true;
+    settings->native_colors = false;
     settings->bell_on_message = 0;
     settings->bell_on_filetrans = 0;
     settings->bell_on_filetrans_accept = 0;
     settings->bell_on_invite = 0;
-    settings->colour_theme = DFLT_COLS;
     settings->history_size = 700;
     settings->notification_timeout = 6000;
-    settings->show_typing_self = SHOW_TYPING_ON;
-    settings->show_typing_other = SHOW_TYPING_ON;
-    settings->show_welcome_msg = SHOW_WELCOME_MSG_ON;
-    settings->show_connection_msg = SHOW_CONNECTION_MSG_ON;
-    settings->show_group_connection_msg = SHOW_GROUP_CONNECTION_MSG_ON;
+    settings->show_typing_self = true;
+    settings->show_typing_other = true;
+    settings->show_welcome_msg = true;
+    settings->show_connection_msg = true;
+    settings->show_group_connection_msg = true;
     settings->nodeslist_update_freq = 1;
     settings->autosave_freq = 600;
 
@@ -157,7 +147,7 @@ static void ui_defaults(Client_Config *settings)
     snprintf(settings->line_normal, LINE_HINT_MAX + 1, "%s", LINE_NORMAL);
     snprintf(settings->line_special, LINE_HINT_MAX + 1, "%s", LINE_SPECIAL);
 
-    settings->mplex_away = MPLEX_ON;
+    settings->mplex_away = true;
     snprintf(settings->mplex_away_note, sizeof(settings->mplex_away_note), "%s", MPLEX_AWAY_NOTE);
 }
 
@@ -172,6 +162,7 @@ static const struct keys_strings {
     const char *page_bottom;
     const char *toggle_peerlist;
     const char *toggle_pastemode;
+    const char *reload_config;
 } key_strings = {
     "keys",
     "next_tab",
@@ -183,6 +174,7 @@ static const struct keys_strings {
     "page_bottom",
     "toggle_peerlist",
     "toggle_paste_mode",
+    "reload_config",
 };
 
 /* defines from toxic.h */
@@ -190,13 +182,14 @@ static void key_defaults(Client_Config *settings)
 {
     settings->key_next_tab = T_KEY_NEXT;
     settings->key_prev_tab = T_KEY_PREV;
-    settings->key_scroll_line_up = KEY_PPAGE;
-    settings->key_scroll_line_down = KEY_NPAGE;
-    settings->key_half_page_up = T_KEY_C_F;
-    settings->key_half_page_down = T_KEY_C_V;
+    settings->key_scroll_line_up = T_KEY_C_F;
+    settings->key_scroll_line_down = T_KEY_C_V;
+    settings->key_half_page_up = KEY_PPAGE;
+    settings->key_half_page_down = KEY_NPAGE;
     settings->key_page_bottom = T_KEY_C_H;
     settings->key_toggle_peerlist = T_KEY_C_B;
     settings->key_toggle_pastemode = T_KEY_C_T;
+    settings->key_reload_config = T_KEY_C_R;
 }
 
 static const struct tox_strings {
@@ -284,12 +277,14 @@ static const struct sound_strings {
 
 static const struct friend_strings {
     const char *self;
+    const char *alias;
     const char *auto_accept_files;
     const char *autolog;
     const char *show_connection_msg;
     const char *tab_name_color;
 } friend_strings = {
     "friends",
+    "alias",
     "auto_accept_files",
     "autolog",
     "show_connection_msg",
@@ -448,7 +443,7 @@ static int settings_init_config(config_t *cfg, const Run_Options *run_opts)
     return 0;
 }
 
-int settings_load_conferences(const Run_Options *run_opts)
+int settings_load_conferences(Windows *windows, const Run_Options *run_opts)
 {
     config_t cfg[1];
     config_init(cfg);
@@ -481,7 +476,7 @@ int settings_load_conferences(const Run_Options *run_opts)
         }
 
         if (config_setting_lookup_string(keys, conference_strings.tab_name_color, &str)) {
-            if (!conference_config_set_tab_name_colour(public_key, str)) {
+            if (!conference_config_set_tab_name_colour(windows, public_key, str)) {
                 fprintf(stderr, "config error: failed to set conference tab name color for %s: (color: %s)\n", public_key, str);
             }
         }
@@ -489,7 +484,7 @@ int settings_load_conferences(const Run_Options *run_opts)
         int autolog_enabled;
 
         if (config_setting_lookup_bool(keys, conference_strings.autolog, &autolog_enabled)) {
-            if (!conference_config_set_autolog(public_key, autolog_enabled != 0)) {
+            if (!conference_config_set_autolog(windows, public_key, autolog_enabled != 0)) {
                 fprintf(stderr, "config error: failed to apply conference autolog setting for %s\n", public_key);
             }
         }
@@ -500,7 +495,7 @@ int settings_load_conferences(const Run_Options *run_opts)
     return 0;
 }
 
-int settings_load_groups(const Run_Options *run_opts)
+int settings_load_groups(Windows *windows, const Run_Options *run_opts)
 {
     config_t cfg[1];
     config_init(cfg);
@@ -533,7 +528,7 @@ int settings_load_groups(const Run_Options *run_opts)
         }
 
         if (config_setting_lookup_string(keys, groupchat_strings.tab_name_color, &str)) {
-            if (!groupchat_config_set_tab_name_colour(public_key, str)) {
+            if (!groupchat_config_set_tab_name_colour(windows, public_key, str)) {
                 fprintf(stderr, "config error: failed to set groupchat tab name color for %s: (color: %s)\n", public_key, str);
             }
         }
@@ -541,7 +536,7 @@ int settings_load_groups(const Run_Options *run_opts)
         int autolog_enabled;
 
         if (config_setting_lookup_bool(keys, groupchat_strings.autolog, &autolog_enabled)) {
-            if (!groupchat_config_set_autolog(public_key, autolog_enabled != 0)) {
+            if (!groupchat_config_set_autolog(windows, public_key, autolog_enabled != 0)) {
                 fprintf(stderr, "config error: failed to apply groupchat autolog setting for %s\n", public_key);
             }
         }
@@ -586,7 +581,8 @@ int settings_load_friends(const Run_Options *run_opts)
 
         if (config_setting_lookup_string(keys, friend_strings.tab_name_color, &str)) {
             if (!friend_config_set_tab_name_colour(public_key, str)) {
-                fprintf(stderr, "config error: failed to set friend tab name color for %s: (color: %s)\n", public_key, str);
+                fprintf(stderr, "config error: failed to set friend tab name color for %s: (color: %s)\n",
+                        public_key, str);
             }
         }
 
@@ -602,7 +598,9 @@ int settings_load_friends(const Run_Options *run_opts)
 
         if (config_setting_lookup_bool(keys, friend_strings.auto_accept_files, &auto_accept_files)) {
             if (!friend_config_set_auto_accept_files(public_key, auto_accept_files != 0)) {
-                fprintf(stderr, "config error: failed to apply friend auto-accept filetransfers setting for: %s\n", public_key);
+                fprintf(stderr,
+                        "config error: failed to apply friend auto-accept filetransfers setting for: %s\n",
+                        public_key);
             }
         }
 
@@ -610,7 +608,15 @@ int settings_load_friends(const Run_Options *run_opts)
 
         if (config_setting_lookup_bool(keys, friend_strings.show_connection_msg, &show_connection_msg)) {
             if (!friend_config_set_show_connection_msg(public_key, show_connection_msg != 0)) {
-                fprintf(stderr, "config error: failed to apply friend show connection message setting for: %s\n", public_key);
+                fprintf(stderr,
+                        "config error: failed to apply friend show connection message setting for: %s\n",
+                        public_key);
+            }
+        }
+
+        if (config_setting_lookup_string(keys, friend_strings.alias, &str)) {
+            if (!friend_config_set_alias(public_key, str, strlen(str))) {
+                fprintf(stderr, "config error: failed to apply alias '%s' for: %s\n", str, public_key);
             }
         }
     }
@@ -644,17 +650,20 @@ int settings_load_main(Client_Config *s, const Run_Options *run_opts)
     }
 
     const char *str = NULL;
+    int bool_val;
 
     /* ui */
     if ((setting = config_lookup(cfg, ui_strings.self)) != NULL) {
-        config_setting_lookup_bool(setting, ui_strings.timestamps, &s->timestamps);
+        if (config_setting_lookup_bool(setting, ui_strings.timestamps, &bool_val)) {
+            s->show_timestamps = bool_val != 0;
+        }
 
         int time = 24;
 
         if (config_setting_lookup_int(setting, ui_strings.time_format, &time)) {
             if (time == 12) {
                 snprintf(s->timestamp_format, sizeof(s->timestamp_format), "%s", "%I:%M %p");
-                snprintf(s->log_timestamp_format, sizeof(s->log_timestamp_format), "%s", "%Y/%m/%d [%I:%M:%S %p]");
+                snprintf(s->log_timestamp_format, sizeof(s->log_timestamp_format), "%s", "%Y/%m/%d [%I:%M %p]");
             }
         }
 
@@ -682,31 +691,57 @@ int settings_load_main(Client_Config *s, const Run_Options *run_opts)
             snprintf(s->log_timestamp_format, sizeof(s->log_timestamp_format), "%s", str);
         }
 
-        config_setting_lookup_bool(setting, ui_strings.alerts, &s->alerts);
-
-        if (config_setting_lookup_bool(setting, ui_strings.bell_on_message, &s->bell_on_message)) {
-            s->bell_on_message = s->bell_on_message ? NT_BEEP : 0;
+        if (config_setting_lookup_bool(setting, ui_strings.alerts, &bool_val)) {
+            s->alerts = bool_val != 0;
         }
 
-        if (config_setting_lookup_bool(setting, ui_strings.bell_on_filetrans, &s->bell_on_filetrans)) {
-            s->bell_on_filetrans = s->bell_on_filetrans ? NT_BEEP : 0;
+        if (config_setting_lookup_bool(setting, ui_strings.show_notification_content, &bool_val)) {
+            s->show_notification_content = bool_val != 0;
         }
 
-        if (config_setting_lookup_bool(setting, ui_strings.bell_on_filetrans_accept, &s->bell_on_filetrans_accept)) {
-            s->bell_on_filetrans_accept = s->bell_on_filetrans_accept ? NT_BEEP : 0;
+        if (config_setting_lookup_bool(setting, ui_strings.bell_on_message, &bool_val)) {
+            s->bell_on_message = bool_val != 0 ? NT_BEEP : 0;
         }
 
-        if (config_setting_lookup_bool(setting, ui_strings.bell_on_invite, &s->bell_on_invite)) {
-            s->bell_on_invite = s->bell_on_invite ? NT_BEEP : 0;
+        if (config_setting_lookup_bool(setting, ui_strings.bell_on_filetrans, &bool_val)) {
+            s->bell_on_filetrans = bool_val != 0 ? NT_BEEP : 0;
         }
 
-        config_setting_lookup_bool(setting, ui_strings.autolog, &s->autolog);
-        config_setting_lookup_bool(setting, ui_strings.native_colors, &s->colour_theme);
-        config_setting_lookup_bool(setting, ui_strings.show_typing_self, &s->show_typing_self);
-        config_setting_lookup_bool(setting, ui_strings.show_typing_other, &s->show_typing_other);
-        config_setting_lookup_bool(setting, ui_strings.show_welcome_msg, &s->show_welcome_msg);
-        config_setting_lookup_bool(setting, ui_strings.show_connection_msg, &s->show_connection_msg);
-        config_setting_lookup_bool(setting, ui_strings.show_group_connection_msg, &s->show_group_connection_msg);
+        if (config_setting_lookup_bool(setting, ui_strings.bell_on_filetrans_accept, &bool_val)) {
+            s->bell_on_filetrans_accept = bool_val != 0 ? NT_BEEP : 0;
+        }
+
+        if (config_setting_lookup_bool(setting, ui_strings.bell_on_invite, &bool_val)) {
+            s->bell_on_invite = bool_val != 0 ? NT_BEEP : 0;
+        }
+
+        if (config_setting_lookup_bool(setting, ui_strings.autolog, &bool_val)) {
+            s->autolog = bool_val != 0;
+        }
+
+        if (config_setting_lookup_bool(setting, ui_strings.native_colors, &bool_val)) {
+            s->native_colors = bool_val != 0;
+        }
+
+        if (config_setting_lookup_bool(setting, ui_strings.show_typing_self, &bool_val)) {
+            s->show_typing_self = bool_val != 0;
+        }
+
+        if (config_setting_lookup_bool(setting, ui_strings.show_typing_other, &bool_val)) {
+            s->show_typing_other = bool_val != 0;
+        }
+
+        if (config_setting_lookup_bool(setting, ui_strings.show_welcome_msg, &bool_val)) {
+            s->show_welcome_msg = bool_val != 0;
+        }
+
+        if (config_setting_lookup_bool(setting, ui_strings.show_connection_msg, &bool_val)) {
+            s->show_connection_msg = bool_val != 0;
+        }
+
+        if (config_setting_lookup_bool(setting, ui_strings.show_group_connection_msg, &bool_val)) {
+            s->show_group_connection_msg = bool_val != 0;
+        }
 
         config_setting_lookup_int(setting, ui_strings.history_size, &s->history_size);
         config_setting_lookup_int(setting, ui_strings.notification_timeout, &s->notification_timeout);
@@ -733,7 +768,9 @@ int settings_load_main(Client_Config *s, const Run_Options *run_opts)
             snprintf(s->line_special, sizeof(s->line_special), "%s", str);
         }
 
-        config_setting_lookup_bool(setting, ui_strings.mplex_away, &s->mplex_away);
+        if (config_setting_lookup_bool(setting, ui_strings.mplex_away, &bool_val)) {
+            s->mplex_away = bool_val != 0;
+        }
 
         if (config_setting_lookup_string(setting, ui_strings.mplex_away_note, &str)) {
             snprintf(s->mplex_away_note, sizeof(s->mplex_away_note), "%s", str);
@@ -748,30 +785,30 @@ int settings_load_main(Client_Config *s, const Run_Options *run_opts)
     if ((setting = config_lookup(cfg, tox_strings.self)) != NULL) {
         if (config_setting_lookup_string(setting, tox_strings.download_path, &str)) {
             snprintf(s->download_path, sizeof(s->download_path), "%s", str);
-            int len = strlen(s->download_path);
+            const size_t len = strlen(s->download_path);
 
             /* make sure path ends with a '/' */
             if (len >= sizeof(s->download_path) - 2) {
                 s->download_path[0] = '\0';
-            } else if (s->download_path[len - 1] != '/') {
+            } else if (len > 0 && s->download_path[len - 1] != '/') {
                 strcat(&s->download_path[len - 1], "/");
             }
         }
 
         if (config_setting_lookup_string(setting, tox_strings.chatlogs_path, &str)) {
             snprintf(s->chatlogs_path, sizeof(s->chatlogs_path), "%s", str);
-            int len = strlen(s->chatlogs_path);
+            const size_t len = strlen(s->chatlogs_path);
 
             if (len >= sizeof(s->chatlogs_path) - 2) {
                 s->chatlogs_path[0] = '\0';
-            } else if (s->chatlogs_path[len - 1] != '/') {
+            } else if (len > 0 && s->chatlogs_path[len - 1] != '/') {
                 strcat(&s->chatlogs_path[len - 1], "/");
             }
         }
 
         if (config_setting_lookup_string(setting, tox_strings.avatar_path, &str)) {
             snprintf(s->avatar_path, sizeof(s->avatar_path), "%s", str);
-            int len = strlen(str);
+            const size_t len = strlen(str);
 
             if (len >= sizeof(s->avatar_path)) {
                 s->avatar_path[0] = '\0';
@@ -782,11 +819,11 @@ int settings_load_main(Client_Config *s, const Run_Options *run_opts)
 
         if (config_setting_lookup_string(setting, tox_strings.autorun_path, &str)) {
             snprintf(s->autorun_path, sizeof(s->autorun_path), "%s", str);
-            int len = strlen(str);
+            const size_t len = strlen(str);
 
             if (len >= sizeof(s->autorun_path) - 2) {
                 s->autorun_path[0] = '\0';
-            } else if (s->autorun_path[len - 1] != '/') {
+            } else if (len > 0 && s->autorun_path[len - 1] != '/') {
                 strcat(&s->autorun_path[len - 1], "/");
             }
         }
@@ -795,7 +832,7 @@ int settings_load_main(Client_Config *s, const Run_Options *run_opts)
 
         if (config_setting_lookup_string(setting, tox_strings.password_eval, &str)) {
             snprintf(s->password_eval, sizeof(s->password_eval), "%s", str);
-            int len = strlen(str);
+            const size_t len = strlen(str);
 
             if (len >= sizeof(s->password_eval)) {
                 s->password_eval[0] = '\0';
@@ -842,6 +879,10 @@ int settings_load_main(Client_Config *s, const Run_Options *run_opts)
         if (config_setting_lookup_string(setting, key_strings.toggle_pastemode, &tmp)) {
             set_key_binding(&s->key_toggle_pastemode, &tmp);
         }
+
+        if (config_setting_lookup_string(setting, key_strings.reload_config, &tmp)) {
+            set_key_binding(&s->key_reload_config, &tmp);
+        }
     }
 
 #ifdef AUDIO
@@ -863,7 +904,9 @@ int settings_load_main(Client_Config *s, const Run_Options *run_opts)
         config_setting_lookup_int(setting, audio_strings.chat_audio_channels, &s->chat_audio_channels);
         s->chat_audio_channels = s->chat_audio_channels <= 0 || s->chat_audio_channels > 2 ? 2 : s->chat_audio_channels;
 
-        config_setting_lookup_bool(setting, audio_strings.push_to_talk, &s->push_to_talk);
+        if (config_setting_lookup_bool(setting, audio_strings.push_to_talk, &bool_val)) {
+            s->push_to_talk = bool_val != 0;
+        }
     }
 
 #endif
@@ -943,3 +986,40 @@ int settings_load_main(Client_Config *s, const Run_Options *run_opts)
     config_destroy(cfg);
     return 0;
 }
+
+void settings_reload(Toxic *toxic)
+{
+    Client_Config *c_config = toxic->c_config;
+    const Run_Options *run_opts = toxic->run_opts;
+    Windows *windows = toxic->windows;
+
+    int ret = settings_load_main(c_config, run_opts);
+
+    if (ret < 0) {
+        fprintf(stderr, "Failed to reload global settings (error %d)\n", ret);
+    }
+
+    friend_reset_default_config_settings(c_config);
+
+    ret = settings_load_friends(run_opts);
+
+    if (ret < 0) {
+        fprintf(stderr, "Failed to reload friend settings (error %d)\n", ret);
+    }
+
+    ret = settings_load_conferences(windows, run_opts);
+
+    if (ret < 0) {
+        fprintf(stderr, "Failed to reload conference settings (error %d)\n", ret);
+    }
+
+    ret = settings_load_groups(windows, run_opts);
+
+    if (ret < 0) {
+        fprintf(stderr, "Failed to reload group settings (error %d)\n", ret);
+    }
+
+    endwin();
+    init_term(c_config, NULL, run_opts->default_locale);
+    refresh_window_names(toxic);
+}
