@@ -1,23 +1,9 @@
 /*  settings.h
  *
+ *  Copyright (C) 2014-2024 Toxic All Rights Reserved.
  *
- *  Copyright (C) 2024 Toxic All Rights Reserved.
- *
- *  This file is part of Toxic.
- *
- *  Toxic is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
- *
- *  Toxic is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with Toxic.  If not, see <http://www.gnu.org/licenses/>.
- *
+ *  This file is part of Toxic. Toxic is free software licensed
+ *  under the GNU General Public License 3.0.
  */
 
 #ifndef SETTINGS_H
@@ -35,31 +21,32 @@
 
 #define PASSWORD_EVAL_MAX 512
 
+typedef struct Toxic Toxic;
+
 /* Holds user setting values defined in the toxic config file. */
 typedef struct Client_Config {
-    int autolog;           /* boolean */
-    int alerts;            /* boolean */
+    bool autolog;
+    bool alerts;
+    bool show_notification_content;
+    bool show_typing_self;
+    bool show_typing_other;
+    bool show_welcome_msg;
+    bool show_connection_msg;
+    bool show_group_connection_msg;
+    bool show_timestamps;
 
-    /* boolean (is set to NT_BEEP or 0 after loading) */
     int bell_on_message;
     int bell_on_filetrans;
     int bell_on_filetrans_accept;
     int bell_on_invite;
 
-    int timestamps;        /* boolean */
     char timestamp_format[TIME_STR_SIZE];
     char log_timestamp_format[TIME_STR_SIZE];
 
-    int colour_theme;      /* boolean (0 for default toxic colours) */
     int history_size;      /* int between MIN_HISTORY and MAX_HISTORY */
     int notification_timeout;
-    int show_typing_self;  /* boolean */
-    int show_typing_other; /* boolean */
-    int show_welcome_msg;  /* boolean */
-    int show_connection_msg;  /* boolean */
-    int show_group_connection_msg;  /* boolean */
-    int nodeslist_update_freq;  /* int (<= 0 to disable updates) */
-    int autosave_freq; /* int (<= 0 to disable autosave) */
+    int nodeslist_update_freq;  /* <= 0 to disable updates */
+    int autosave_freq; /* <= 0 to disable autosave */
 
     char line_join[LINE_HINT_MAX + 1];
     char line_quit[LINE_HINT_MAX + 1];
@@ -73,6 +60,7 @@ typedef struct Client_Config {
     char autorun_path[PATH_MAX];
     char password_eval[PASSWORD_EVAL_MAX];
 
+    bool native_colors;
     char color_bar_bg[COLOR_STR_SIZE];
     char color_bar_fg[COLOR_STR_SIZE];
     char color_bar_accent[COLOR_STR_SIZE];
@@ -87,8 +75,9 @@ typedef struct Client_Config {
     int key_page_bottom;
     int key_toggle_peerlist;
     int key_toggle_pastemode;
+    int key_reload_config;
 
-    int mplex_away; /* boolean (1 for reaction to terminal attach/detach) */
+    bool mplex_away; /*  true for reaction to terminal attach/detach */
     char mplex_away_note [TOX_MAX_STATUS_MESSAGE_LENGTH];
     char group_part_message[TOX_GROUP_MAX_PART_LENGTH];
 
@@ -98,50 +87,21 @@ typedef struct Client_Config {
     double VAD_threshold;
     int conference_audio_channels;
     int chat_audio_channels;
-    int push_to_talk;      /* boolean */
+    bool push_to_talk;
 #endif
 } Client_Config;
 
-enum settings_values {
-    AUTOLOG_OFF = 0,
-    AUTOLOG_ON = 1,
-
-    TIMESTAMPS_OFF = 0,
-    TIMESTAMPS_ON = 1,
-
-    ALERTS_DISABLED = 0,
-    ALERTS_ENABLED = 1,
-
-    DFLT_COLS = 0,
-    NATIVE_COLS = 1,
-
-    SHOW_TYPING_OFF = 0,
-    SHOW_TYPING_ON = 1,
-
-    SHOW_WELCOME_MSG_OFF = 0,
-    SHOW_WELCOME_MSG_ON = 1,
-
-    SHOW_CONNECTION_MSG_OFF = 0,
-    SHOW_CONNECTION_MSG_ON = 1,
-
-    SHOW_GROUP_CONNECTION_MSG_OFF = 0,
-    SHOW_GROUP_CONNECTION_MSG_ON = 1,
-
-    DFLT_HST_SIZE = 700,
-
-    MPLEX_OFF = 0,
-    MPLEX_ON = 1,
-};
-
 #define LINE_JOIN    "-->"
 #define LINE_QUIT    "<--"
 #define LINE_ALERT   "-!-"
 #define LINE_NORMAL  "-"
 #define LINE_SPECIAL ">"
 #define TIMESTAMP_DEFAULT      "%H:%M"
-#define LOG_TIMESTAMP_DEFAULT  "%Y/%m/%d [%H:%M:%S]"
+#define LOG_TIMESTAMP_DEFAULT  "%Y/%m/%d [%H:%M]"
 #define MPLEX_AWAY_NOTE "Away from keyboard, be back soon!"
 
+typedef struct Windows Windows;
+
 /*
  * Loads the config file into `run_opts` and creates an empty file if it does not
  * already exist. This function must be called before any other `settings_load` function.
@@ -192,7 +152,7 @@ int settings_load_friends(const Run_Options *run_opts);
  *
  * This function will have no effect on groupchat instances that are created in the future.
  */
-int settings_load_groups(const Run_Options *run_opts);
+int settings_load_groups(Windows *windows, const Run_Options *run_opts);
 
 /*
  * Loads conference config settings from the toxic config file pointed to by `patharg`.
@@ -204,6 +164,11 @@ int settings_load_groups(const Run_Options *run_opts);
  *
  * This function will have no effect on conference instances that are created in the future.
  */
-int settings_load_conferences(const Run_Options *run_opts);
+int settings_load_conferences(Windows *windows, const Run_Options *run_opts);
+
+/*
+ * Reloads config settings.
+ */
+void settings_reload(Toxic *toxic);
 
 #endif /* SETTINGS_H */
